#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Checks if Python dependencies for product demo kit are satisfied
#  =============================================================================

import os
import sys
import sqlite3
import gxd_functions as gxd

python = sys.argv[1]
database = sys.argv[2]

if database == 'SQLITE':
	try:
		import sqlite3
	except ImportError:
		gxd.error("\t{0}: Python ({1}) module sqlite3 is not installed. Consider using latest Python 3.x or installing sqlite3 module\n".format( os.path.basename(__file__), python ))
		sys.exit(1)

sqlite_version_min = (3,14)		# Bundled with Python 3.6+?

if sys.version_info[0] == 2:

	if sys.version_info[:2] < (2,7):
		gxd.error("\t{}: At least Python 2.7.x required, latest Python 3.x strongly recommended\n".format(os.path.basename(__file__)))
		sys.exit(1)

	if database == 'SQLITE' and sqlite3.sqlite_version_info < (3,14):
		try:
			from pysqlite2 import dbapi2 as sqlite
			if sqlite.sqlite_version_info < sqlite_version_min:
				gxd.error("\t{0}: Python ({1}) built-in sqlite3 and pysqlite versions are < {2}. Consider using latest Python 3.x or upgrading system sqlite libraries\n".format( os.path.basename(__file__), python, ",".join(map(str, sqlite_version_min)) ))
				sys.exit(1)			
		except ImportError:
			gxd.error("\t{0}: Python ({1}) built-in sqlite3 version is < {2}. Consider using latest Python 3.x, upgrading system sqlite libraries or installing pysqlite module\n".format( os.path.basename(__file__), python, ",".join(map(str, sqlite_version_min)) ))
			sys.exit(1)

	try:
		import contextlib2
	except ImportError:
		gxd.error("\t{0}: Python ({1}) module contextlib2 is not installed. Consider using latest Python 3.x or installing contextlib2 module\n".format( os.path.basename(__file__), python ))
		sys.exit(1)

elif sys.version_info[0] == 3:

	if sys.version_info[:2] < (3,3):
		gxd.error("\t{}: At least Python 3.3.x required, latest Python 3.x strongly recommended \n".format(os.path.basename(__file__)))
		sys.exit(1)

	if database == 'SQLITE' and sqlite3.sqlite_version_info < (3,14):
			gxd.error("\t{0}: Python ({1}) built-in sqlite3 version is < {2}. Consider using latest Python 3.x\n".format( os.path.basename(__file__), python, ",".join(map(str, sqlite_version_min)) ))
			sys.exit(1)

	try:
		import contextlib
	except ImportError:
		gxd.error("\t{0}: Python ({1}) module contextlib is not installed. Consider using latest Python 3.x\n".format( os.path.basename(__file__), python ))
		sys.exit(1)


if database == 'PGSQL':
	try:
		import psycopg2
	except ImportError:
		gxd.error("\t{0}: Python ({1}) Postgres DB API module psycopg2 is not installed. Please refer to readme file for resolving Postgres dependencies.\n".format( os.path.basename(__file__), python ))
		sys.exit(1)

if database == 'MYSQL':
	try:
		import mysql.connector
	except ImportError:
		gxd.error("\t{0}: Python ({1}) MySQL DB API module mysql.connector is not installed. Please refer to readme file for resolving MySQL dependencies.\n".format( os.path.basename(__file__), python ))
		sys.exit(1)

if database == 'MONGO':
	try:
		import pymongo
		import decimal
		import bson
	except ImportError:
		gxd.error("\t{0}: One or more of Python ({1}) MongoDB API modules is not installed: pymongo, decimal, bson. Please refer to readme file for resolving Mongo dependencies.\n".format( os.path.basename(__file__), python ))
		sys.exit(1)
