#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Executes given SQL file against configured MySQL database using DBAPI.
#  Commits either entire file or each statement, given a configurable separator.
#  SQL statement can contain Python-like parameters {0},{1},...
#  Parameters values can be passed via pipe-separated string "value0|value1|..."
#  =============================================================================

import os
import sys
import mysql.connector

sys.path.append(os.path.join(sys.path[0],'..'))
import gxd_functions as gxd

# SQL file to run against the database
sql_file = sys.argv[1]

# Empty filename means read from stdin
if sql_file == "":
	sql_file = 0

# Read values for SQL parameters
if len(sys.argv) > 2:
	statement_params = sys.argv[2].split('|')
else:
	statement_params = []

# Commit each statement or entire file
if len(sys.argv) > 3 and sys.argv[3].upper() == 'TRUE':
	commit_each_statement = True
else:
	commit_each_statement = False

# Configure statment separator
if len(sys.argv) > 4:
	statement_separator = sys.argv[4]
else:
	statement_separator = ';'

# Set flag if result is requested
if len(sys.argv) > 5:
	result_requested = sys.argv[5].upper()
else:
	result_requested = None

gxd_dir 	= os.environ['GXD_PDK_DIR']
custom_conf = os.environ['GXD_PDK_CONFIG']

mysql_host 		= gxd.get_config_setting('MYSQL_HOST', gxd_dir, custom_conf)
mysql_port 		= gxd.get_config_setting('MYSQL_PORT', gxd_dir, custom_conf)
mysql_user 		= gxd.get_config_setting('MYSQL_USER', gxd_dir, custom_conf)
mysql_pass 		= gxd.get_config_setting('MYSQL_PASS', gxd_dir, custom_conf)


# Check if server-version-specific SQL file exists and if so, use that
def versioned_sql_if_any(db, sql_file):
	if sql_file == 0:
		return 0
	major_version = db.get_server_info().split('.')[0]
	punctuated_fn = sql_file.split('.')
	punctuated_fn[-2] += "_v{}".format(major_version)	# this assumes SQL file has an extension
	if os.path.exists('.'.join(punctuated_fn)):
		return '.'.join(punctuated_fn)
	else:
		return sql_file


# Optionally print one or all returned rows to stdout
def optional_output(cursor, result_requested):
	if result_requested == 'FETCH_ONE':
		row = cursor.fetchone()
		if row is not None:
			print( "\t".join([str(value) for value in list(row)]) )
	if result_requested == 'FETCH_ALL':
		rows = cursor.fetchall()
		if rows is not None:
			for row in rows:
				print( "\t".join([str(value) for value in list(row)]) )


try:
	db = mysql.connector.connect( user=mysql_user, password=mysql_pass, host=mysql_host, port=mysql_port )
	sql_file = versioned_sql_if_any(db, sql_file)
	cursor = db.cursor()
	for statement in gxd.sql_iterator(sql_file, statement_separator):

		for i, param in enumerate(statement_params):
			statement = statement.replace('{' + str(i) + '}', param)
		try:
			#sys.stderr.write(statement)
			cursor.execute(statement)
			if commit_each_statement:
				optional_output(cursor, result_requested)
				db.commit()

		except mysql.connector.Error as e:
			gxd.error("\t{0}: DBAPI exception executing statement:\n\t{1}\n\t{2}\n".format( os.path.basename(__file__), statement, str(e) ))
			sys.exit(1)

	if not commit_each_statement:
		optional_output(cursor, result_requested)
		db.commit()

except mysql.connector.Error as e:
	gxd.error("\t{0}: DBAPI exception: {1} connecting or committing to database\n".format( os.path.basename(__file__), str(e) ))
	sys.exit(1)
