#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Executes given SQL file against configured Postgres database using DBAPI.
#  Commits either entire file or each statement, given a configurable separator.
#  SQL statement can contain Python-like parameters {0},{1},...
#  Parameters values can be passed via pipe-separated string "value0|value1|..."
#  =============================================================================

import os
import sys
import psycopg2

sys.path.append(os.path.join(sys.path[0],'..'))
import gxd_functions as gxd

# SQL file to run against the database
sql_file = sys.argv[1]

# Empty filename means read from stdin
if sql_file == "":
	sql_file = 0

# Read values for SQL parameters
if len(sys.argv) > 2:
	statement_params = sys.argv[2].split('|')
else:
	statement_params = []

# Commit each statement or entire file
if len(sys.argv) > 3 and sys.argv[3].upper() == 'TRUE':
	commit_each_statement = True
else:
	commit_each_statement = False

# Configure statment separator
if len(sys.argv) > 4:
	statement_separator = sys.argv[4]
else:
	statement_separator = ';'

# Set flag if result is requested
if len(sys.argv) > 5:
	result_requested = sys.argv[5].upper()
else:
	result_requested = None

gxd_dir 	= os.environ['GXD_PDK_DIR']
custom_conf = os.environ['GXD_PDK_CONFIG']

pgsql_host 		= gxd.get_config_setting('PGSQL_HOST', gxd_dir, custom_conf)
pgsql_port 		= gxd.get_config_setting('PGSQL_PORT', gxd_dir, custom_conf)
pgsql_user 		= gxd.get_config_setting('PGSQL_USER', gxd_dir, custom_conf)
pgsql_pass 		= gxd.get_config_setting('PGSQL_PASS', gxd_dir, custom_conf)
pgsql_db_name 	= gxd.get_config_setting('PGSQL_DBNAME', gxd_dir, custom_conf)


# Optionally print one or all returned rows to stdout
def optional_output(cursor, result_requested):
	if result_requested == 'FETCH_ONE':
		row = cursor.fetchone()
		if row is not None:
			print( "\t".join([str(value) for value in list(row)]) )
	if result_requested == 'FETCH_ALL':
		rows = cursor.fetchall()
		if rows is not None:
			for row in rows:
				print( "\t".join([str(value) for value in list(row)]) )


try:
	db = psycopg2.connect(host = pgsql_host, port = pgsql_port, user = pgsql_user, password = pgsql_pass, database = pgsql_db_name)

	cursor = db.cursor()
	for statement in gxd.sql_iterator(sql_file, statement_separator):

		for i, param in enumerate(statement_params):
			statement = statement.replace('{' + str(i) + '}', param)
		try:
			#sys.stderr.write(statement)
			cursor.execute(statement)
			if commit_each_statement:
				db.commit()
				optional_output(cursor, result_requested)

		except psycopg2.Error as e:
			gxd.error("\t{0}: DBAPI exception executing statement:\n\t{1}\n\t{2}\n".format( os.path.basename(__file__), statement, str(e) ))
			sys.exit(1)

	if not commit_each_statement:
		db.commit()
		optional_output(cursor, result_requested)

except psycopg2.Error as e:
	gxd.error("\t{0}: DBAPI exception: {1} connecting or committing to database\n".format( os.path.basename(__file__), str(e) ))
	sys.exit(1)
