#!/bin/sh

#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Staging sequence for SQLite
#  =============================================================================

. $GXD_PDK_DIR/bin/functions.sh
load_config

VARS=$GXD_PDK_DIR/vars
echo "JOB_PHASE=STAGE" > $VARS/job_phase.var

SCHEMA_STAGE=${SCHEMA_PREFIX}_stage
SCHEMA_CORE=${SCHEMA_PREFIX}_core

echo

# Check which step the script failed on during previous run, if any
# If it is not 0, we'll skip to the failed step
JOB_STATUS=$1
if  [ $JOB_STATUS = "ERROR" ] &&
	[ -e $VARS/job_phase_step.var ]
then
	. $VARS/job_phase_step.var
else
	JOB_PHASE_STEP=0
fi

TRUTH_VALUE="TRUE"

# Retrieve upstream decision to update or replace core
if [ -e $VARS/core_action.var ]; then
	. $VARS/core_action.var	
else
	CORE_ACTION="REPLACE"
fi


# 1. Create indexes
if [ $JOB_PHASE_STEP -le 1 ]; then
	echo "JOB_PHASE_STEP=1" > $VARS/job_phase_step.var

	msg_ok "STAGE: $PYTHON db_sqlite/runsql.py db/sqlite/stage/create_indexes.sql $SCHEMA_STAGE"
	$PYTHON $GXD_PDK_DIR/bin/db_sqlite/runsql.py $GXD_PDK_DIR/db/sqlite/stage/create_indexes.sql $SCHEMA_STAGE
	if [ $? != 0 ]; then
		msg_err "STAGE: ERROR: $PYTHON db_sqlite/runsql.py db/sqlite/stage/create_indexes.sql $SCHEMA_STAGE returned error"
		exit_abnormal
	fi
fi

# 2. De-duplicate objects with multiple updates per object (possibility with longer update runs)
if [ $JOB_PHASE_STEP -le 2 ]; then
	echo "JOB_PHASE_STEP=2" > $VARS/job_phase_step.var

	msg_ok "STAGE: $PYTHON db_sqlite/runsql.py db/sqlite/stage/delete_objects_not_having_max_updateid.sql $SCHEMA_STAGE"
	$PYTHON $GXD_PDK_DIR/bin/db_sqlite/runsql.py $GXD_PDK_DIR/db/sqlite/stage/delete_objects_not_having_max_updateid.sql $SCHEMA_STAGE
	if [ $? != 0 ]; then
		msg_err "STAGE: ERROR: $PYTHON db_sqlite/runsql.py db/sqlite/stage/delete_objects_not_having_max_updateid.sql $SCHEMA_STAGE returned error"
		exit_abnormal
	fi

	# File products only: de-dupe by resource_id
	if [ "$FILE_CONFIGS" != "" ]; then
		msg_ok "STAGE: $PYTHON db_sqlite/runsql.py db/sqlite/stage/delete_objects_not_having_min_resourceid.sql $SCHEMA_STAGE"
		$PYTHON $GXD_PDK_DIR/bin/db_sqlite/runsql.py $GXD_PDK_DIR/db/sqlite/stage/delete_objects_not_having_min_resourceid.sql $SCHEMA_STAGE
		if [ $? != 0 ]; then
			msg_err "STAGE: ERROR: $PYTHON db_sqlite/runsql.py db/sqlite/stage/delete_objects_not_having_min_resourceid.sql $SCHEMA_STAGE returned error"
			exit_abnormal
		fi
	fi
fi

# 3. De-duplicate tables using MD5 to eliminate the following
#    - rare timing-related entity duplicates from the API (same object_id, same update_id)
#    - potentially extraneous source data (can occur in some child tables such as program_release*)
if [ $JOB_PHASE_STEP -le 3 ]; then
	echo "JOB_PHASE_STEP=3" > $VARS/job_phase_step.var

	msg_ok "STAGE: $PYTHON db_sqlite/runsql.py db/sqlite/stage/delete_row_hash_duplicates.sql $SCHEMA_STAGE"
	$PYTHON $GXD_PDK_DIR/bin/db_sqlite/runsql.py $GXD_PDK_DIR/db/sqlite/stage/delete_row_hash_duplicates.sql $SCHEMA_STAGE
	if [ $? != 0 ]; then
		msg_err "STAGE: ERROR: $PYTHON db_sqlite/runsql.py db/sqlite/stage/delete_row_hash_duplicates.sql $SCHEMA_STAGE returned error"
		exit_abnormal
	fi
fi

# 4. Some API endpoints/objects can be deconstructed into more than one entity, as per database schema
#    The API will typically provide delete or inactivation signal at the main entity level without replicating it at the dependent entity level, 
#    or omit the dependent entities entirely from the update and only provide a "shell" record for the main entity with the delete or inactive flag.
#    As well, the API object update can implicitly signal deletion of dependent entities when they are omitted from the main entity object update
#    This step reconstructs explicit entity deletion or inactivation signal, based on the diff of the core vs stage table relating the entities of the deconstructed object
#    Objects having both inactive and deleted flags will only have the deleted flag reconstructed in the dependent object
if [ $JOB_PHASE_STEP -le 4 ]; then
	echo "JOB_PHASE_STEP=4" > $VARS/job_phase_step.var

	if [ $CORE_ACTION = "UPDATE" ]; then
		msg_ok "STAGE: $PYTHON db_sqlite/runsql.py db/sqlite/stage/reconstruct_dependent_signals.sql '$SCHEMA_STAGE|$SCHEMA_CORE|$TRUTH_VALUE'"
		$PYTHON $GXD_PDK_DIR/bin/db_sqlite/runsql.py $GXD_PDK_DIR/db/sqlite/stage/reconstruct_dependent_signals.sql "$SCHEMA_STAGE|$SCHEMA_CORE|$TRUTH_VALUE"
		if [ $? != 0 ]; then
			msg_err "STAGE: ERROR: $PYTHON db_sqlite/runsql.py db/sqlite/stage/reconstruct_dependent_signals.sql '$SCHEMA_STAGE|$SCHEMA_CORE|$TRUTH_VALUE' returned error"
			exit_abnormal
		fi
	fi
fi

# 5. Optionally, clear MD5 columns to reduce core storage requirements
if [ $JOB_PHASE_STEP -le 5 ]; then
	echo "JOB_PHASE_STEP=5" > $VARS/job_phase_step.var

	if [ "$CLEAR_MD5" = "TRUE" ]; then
		msg_ok "STAGE: $PYTHON db_sqlite/runsql.py db/sqlite/stage/clear_row_hash.sql $SCHEMA_STAGE"
		$PYTHON $GXD_PDK_DIR/bin/db_sqlite/runsql.py $GXD_PDK_DIR/db/sqlite/stage/clear_row_hash.sql $SCHEMA_STAGE TRUE
		if [ $? != 0 ]; then
			msg_err "STAGE: ERROR: $PYTHON db_sqlite/runsql.py db/sqlite/stage/clear_row_hash.sql $SCHEMA_STAGE returned error"
			exit_abnormal
		fi
	fi
fi

# Once all steps ran successfully, clear previously failed step (if any)
rm -f $VARS/job_phase_step.var

echo
