#!/bin/sh

#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Download and download error recovery for various delivery configurations
#  =============================================================================

. $GXD_PDK_DIR/bin/functions.sh
load_config

VARS=$GXD_PDK_DIR/vars
echo "JOB_PHASE=DOWNLOAD" > $VARS/job_phase.var

echo 

# Check which step the script failed on during previous run, if any
# If it is not empty, we'll go to the failed step
JOB_STATUS=$1
if  [ $JOB_STATUS = "ERROR" ] &&
	[ -e $VARS/job_phase_step.var ]
then
	. $VARS/job_phase_step.var
else
	JOB_PHASE_STEP=""
fi

# Optional parameters passed by the main run script
DELIVERY_OVERRIDE=$2
CORE_ACTION_OVERRIDE=$3
FILE_INGEST_TYPE=$4

# If override parameters were given, update config parameters
if [ -n "$DELIVERY_OVERRIDE" ]; then
	DELIVERY=$DELIVERY_OVERRIDE
fi


# Previous job succeeded (or first start)
if [ $JOB_STATUS = "OK" ]; then

	# API - default
	if [ $DELIVERY = "API" ]; then 

		if [ -e $VARS/last_good_ingest_id.var ]; then
			echo "CORE_ACTION=UPDATE" > $VARS/core_action.var
		else
			echo "CORE_ACTION=REPLACE" > $VARS/core_action.var
		fi

		$GXD_PDK_DIR/bin/download_api.sh
		if [ $? != 0 ]; then
			exit_abnormal
		fi

	# Seed files
	elif [ $DELIVERY = "SEED" ] ||
		 [ $DELIVERY = "SEED_OR_API" ]; then 

		# Remember most recent seed file ingest (replacement) date
		REPLACE_DATE=0
		if [ -e $VARS/replace_date.var ]; then
		    . $VARS/replace_date.var
		fi
		PREV_REPLACE_DATE=$REPLACE_DATE

		# Set core action and invoke seed download
		echo "CORE_ACTION=REPLACE" > $VARS/core_action.var
		$GXD_PDK_DIR/bin/download_seed.sh
		if [ $? != 0 ]; then
			exit_abnormal
		fi

		# Compare REPLACE_DATEs and check if we have new seed files
		REPLACE_DATE=0
		if [ -e $VARS/replace_date.var ]; then
			. $VARS/replace_date.var
		fi
		
		# If no new seed files, we're done (SEED) - or fall back to API update (SEED_OR_API)
		if [ $REPLACE_DATE = $PREV_REPLACE_DATE ]; then

			if [ $DELIVERY = "SEED" ]; then
				# Rather than replacing DB with blank data, let the main script proceed with update instead
				echo "CORE_ACTION=UPDATE" > $VARS/core_action.var


			elif [ $DELIVERY = "SEED_OR_API" ]; then
				
				# Fall back to API update
				echo "CORE_ACTION=UPDATE" > $VARS/core_action.var
				$GXD_PDK_DIR/bin/download_api.sh
				if [ $? != 0 ]; then
					exit_abnormal
				fi
			fi
		fi

	# File products
	elif [ $DELIVERY = "FTP" ]; then 

		# If ingest type was given, store it for further usage by the download_ftp.sh script
		rm -f $VARS/file_ingest_type.var
		if [ -n "$FILE_INGEST_TYPE" ]; then
			echo "FILE_INGEST_TYPE=$FILE_INGEST_TYPE" > $VARS/file_ingest_type.var
		fi

		$GXD_PDK_DIR/bin/download_ftp.sh
		if [ $? != 0 ]; then
			exit_abnormal
		fi

	elif [ $DELIVERY = "NONE" ]; then 
		# Test option; rely on having last good ingest or explicit override
		if [ -e $VARS/last_good_ingest_id.var ]; then
			echo "CORE_ACTION=UPDATE" > $VARS/core_action.var
		else
			echo "CORE_ACTION=REPLACE" > $VARS/core_action.var
		fi
		# If core action override was given, use that
		if [ -n "$CORE_ACTION_OVERRIDE" ]; then
			echo "CORE_ACTION=$CORE_ACTION_OVERRIDE" > $VARS/core_action.var
		fi		
	fi


# Previous job failed in download phase
elif [ $JOB_STATUS = "ERROR" ]; then

	# Previous seed download failed, rerun
	if  [ $JOB_PHASE_STEP = "SEED" ]; then

		# Remember most recent seed file ingest (replacement) date
		REPLACE_DATE=0
		if [ -e $VARS/replace_date.var ]; then
		    . $VARS/replace_date.var
		fi
		PREV_REPLACE_DATE=$REPLACE_DATE

		# Invoke seed download
		echo "CORE_ACTION=REPLACE" > $VARS/core_action.var
		$GXD_PDK_DIR/bin/download_seed.sh
		if [ $? != 0 ]; then
			exit_abnormal
		fi

		# Compare and check if we have new seed files
		REPLACE_DATE=0
		if [ -e $VARS/replace_date.var ]; then
		    . $VARS/replace_date.var
		fi		
		
		# If no new seed files, we're done 
		if [ $REPLACE_DATE = $PREV_REPLACE_DATE ]; then
			# Rather than replacing DB with blank data, let the main script proceed with update instead
			echo "CORE_ACTION=UPDATE" > $VARS/core_action.var
		fi

	# Previous API download failed, rerun
	elif [ $JOB_PHASE_STEP = "API" ]; then
		$GXD_PDK_DIR/bin/download_api.sh
		if [ $? != 0 ]; then
			exit_abnormal
		fi

	# Previous FTP download failed, rerun
	elif [ $JOB_PHASE_STEP = "FTP" ]; then
		$GXD_PDK_DIR/bin/download_ftp.sh
		if [ $? != 0 ]; then
			exit_abnormal
		fi

	fi
fi

# Once steps ran successfully, clear previously failed step (if any)
rm -f $VARS/job_phase_step.var

echo

