#!/bin/sh

#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Iterates over resources (endpoints/filenames) in the respective config file
#  For each resource:
#  Generates a list of XML files and calls parse_filelist.sh
#  After XML files are parsed into corresponding TSV and/or JSONL files,
#  merges multiple TSV files into a single file for each database table
#  and/or merges multiple JSONL files into a single file for each entity
#  =============================================================================

. $GXD_PDK_DIR/bin/functions.sh
load_config

VARS=$GXD_PDK_DIR/vars
mkdir -p $GXD_PDK_DIR/files/tsv
mkdir -p $GXD_PDK_DIR/files/jsonl
echo "JOB_PHASE=PARSE" > $VARS/job_phase.var

echo

# Determine whether resources are represented by endpoints (API products) or files (FTP products)
if [ "$FILE_CONFIGS" != "" ]; then
	CONFIGS=$FILE_CONFIGS
elif [ "$ENDPOINT_CONFIGS" != "" ]; then
	CONFIGS=$ENDPOINT_CONFIGS
fi

# If the respective config file exists
if [ -e $GXD_PDK_DIR/config/$CONFIGS ]; then

	while read -r CONFIG; do

		RESOURCE=$( echo "$CONFIG" | cut -f1 )

		# For files, the parser name could be different from the resource name
		if [ "$FILE_CONFIGS" != "" ]; then
			PARSER=$( echo "$CONFIG" | cut -f2 )
		else
			PARSER=$RESOURCE
		fi

		msg_ok "PARSE: Parsing $RESOURCE"

		# Create dir for output TSVs
		GXD_RESOURCE_TSV_DIR=$GXD_PDK_DIR/files/tsv/$RESOURCE
		mkdir -p $GXD_RESOURCE_TSV_DIR

		# Create dir for output JSONL
		GXD_RESOURCE_JSONL_DIR=$GXD_PDK_DIR/files/jsonl/$RESOURCE
		mkdir -p $GXD_RESOURCE_JSONL_DIR


		# Check if resource XML files are available and call parser if so
		XML_COUNT=$( find $GXD_PDK_DIR/files/xml -type f -name "$RESOURCE-*.xml" -maxdepth 1 | wc -l )
		if [ $XML_COUNT != 0 ]; then

			# List all XML files for the resource and pipe them to the filelist parsing script
			# Future: parallelize using "split -n l/i/n" to separate filelist into n parts and pipe i-th part to a separate process
			msg_ok "PARSE: find files/xml -type f -name '$RESOURCE-*.xml' -maxdepth 1 | xargs basename -a |  parse_filelist.sh '' '$RESOURCE' '$PARSER'"
			find $GXD_PDK_DIR/files/xml -type f -name "$RESOURCE-*.xml" -maxdepth 1 | xargs basename -a | $GXD_PDK_DIR/bin/parse_filelist.sh "" "$RESOURCE" "$PARSER"
			if [ $? != 0 ]; then
				msg_err "PARSE: ERROR: find '$RESOURCE-*.xml' or parse_filelist.sh '' '$RESOURCE' '$PARSER' returned an error"
				exit_abnormal
			fi

			# Future: wait for parallel processes spawned above to complete
			# wait

		else
			msg_ok "PARSE: No XML files available for $RESOURCE - endpoint/resource might be up to date"
		fi


		# Enumerate all distinct output TSV tables for the resource
		# There may be none if no resources were available and/or there may be leftovers from previous job if it failed
		TSV_TABLES=$( find $GXD_RESOURCE_TSV_DIR -type f -name '*.tsv' | xargs basename -a | cut -d'-' -f1 | sort | uniq )
		if [ $? != 0 ]; then
			msg_err "PARSE: ERROR: unable to build a list of TSV tables"
			exit_abnormal
		fi

		for TABLE in $TSV_TABLES; do

			# Check if table TSV files are available and if not, skip to next table
			TSV_COUNT=$( find $GXD_RESOURCE_TSV_DIR -type f -name "$TABLE-*.tsv" | wc -l )
			if [ $TSV_COUNT = 0 ]; then
				msg_ok "PARSE: No TSV files available to merge for $RESOURCE/$TABLE"
				continue
			fi

			# Merge TSVs with different update_ids into single TSV file, sort by update_id
			msg_ok "PARSE: Merging files/tsv/$RESOURCE/$TABLE-*.tsv into $TABLE.tsv"
			find $GXD_RESOURCE_TSV_DIR -type f -name "$TABLE-*.tsv" | sort -n -k2 -t'-' | xargs cat > $GXD_RESOURCE_TSV_DIR/$TABLE.tsv
			if [ $? != 0 ]; then
				mv $GXD_RESOURCE_TSV_DIR/$TABLE.tsv $GXD_LOG_DIR/errors/
				msg_err "PARSE: ERROR: unable to merge $TABLE-*.tsv files"
				exit_abnormal
			fi
			find $GXD_RESOURCE_TSV_DIR -type f -name "$TABLE-*.tsv" -delete

			# Append to or overwrite existing TSVs (if any) in files/tsv directory
			if [ $APPEND_TSV = "TRUE" ]; then
				msg_ok "PARSE: Appending files/tsv/$TABLE.tsv"
				cat $GXD_RESOURCE_TSV_DIR/$TABLE.tsv >> $GXD_PDK_DIR/files/tsv/$TABLE.tsv
				rm $GXD_RESOURCE_TSV_DIR/$TABLE.tsv
			else
				msg_ok "PARSE: Writing files/tsv/$TABLE.tsv"
				mv $GXD_RESOURCE_TSV_DIR/$TABLE.tsv $GXD_PDK_DIR/files/tsv/$TABLE.tsv
			fi

		done

		# Enumerate all distinct output JSONL entities for the endpoint
		# There may be none if endpoint is up to date and/or there may be leftovers from previous job if it failed
		JSONL_ENTITIES=$( find $GXD_RESOURCE_JSONL_DIR -type f -name '*.jsonl' | xargs basename -a | cut -d'-' -f1 | sort | uniq )
		if [ $? != 0 ]; then
			msg_err "PARSE: ERROR: unable to build a list of JSONL entities"
			exit_abnormal
		fi

		for ENTITY in $JSONL_ENTITIES; do

			# Check if entity JSONL files are available and if not, skip to next entity
			JSONL_COUNT=$( find $GXD_RESOURCE_JSONL_DIR -type f -name "$ENTITY-*.jsonl" | wc -l )
			if [ $JSONL_COUNT = 0 ]; then
				msg_ok "PARSE: No JSONL files available to merge for $RESOURCE/$ENTITY"
				continue
			fi

			# Merge JSONL with different update_ids into single JSONL file, sort by update_id
			msg_ok "PARSE: Merging files/jsonl/$RESOURCE/$ENTITY-*.jsonl into $ENTITY.jsonl"
			find $GXD_RESOURCE_JSONL_DIR -type f -name "$ENTITY-*.jsonl" | sort -n -k2 -t'-' | xargs cat > $GXD_RESOURCE_JSONL_DIR/$ENTITY.jsonl
			if [ $? != 0 ]; then
				mv $GXD_RESOURCE_JSONL_DIR/$ENTITY.jsonl $GXD_LOG_DIR/errors/
				msg_err "PARSE: ERROR: unable to merge $ENTITY-*.jsonl files"
				exit_abnormal
			fi
			find $GXD_RESOURCE_JSONL_DIR -type f -name "$ENTITY-*.jsonl" -delete

			# Append to or overwrite existing JSONL (if any) in files/jsonl directory
			if [ $APPEND_TSV = "TRUE" ]; then
				msg_ok "PARSE: Appending files/jsonl/$ENTITY.jsonl"
				cat $GXD_RESOURCE_JSONL_DIR/$ENTITY.jsonl >> $GXD_PDK_DIR/files/jsonl/$ENTITY.jsonl
				rm $GXD_RESOURCE_JSONL_DIR/$ENTITY.jsonl
			else
				msg_ok "PARSE: Writing files/jsonl/$ENTITY.jsonl"
				mv $GXD_RESOURCE_JSONL_DIR/$ENTITY.jsonl $GXD_PDK_DIR/files/jsonl/$ENTITY.jsonl
			fi

		done

		echo	
	done < $GXD_PDK_DIR/config/$CONFIGS

else
	msg_ok "PARSE: No resources to parse specified"
fi
