#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Parses a given ON Lineups XML file
#  Validates each record against fields configuration file
#  Writes validated records into TSV files for the endpoint
#  =============================================================================

import os
import sys

sys.path.append(os.path.join(sys.path[0],'..'))
import gxd_functions as gxd

xml_file 	= sys.argv[1]
files_path 	= sys.argv[2]

class GnLineupsParser(gxd.GnEndpointParser):

	def parse(self, xml_file):

		for l in self.xml_nodes(xml_file, "on/lineups/lineup"):

			lineup = {}

			lineupId 	= self.check( l, 'id' )
			updateId 	= self.check( l, 'updateId' )
			updateDate 	= self.check( l, 'updateDate' )

			lineup['lineupId'] 	= lineupId
			lineup['updateId'] 	= updateId
			lineup['updateDate'] = updateDate

			lineup['deleted'] 	= self.format_boolean( self.check( l, 'deleted' ) )			
			lineup['country'] 	= self.check( l.find('lineupInfo/country') )
			lineup['countryId'] = self.check( l.find('lineupInfo/country'), 'countryId' )
			lineup['mso'] 		= self.check( l.find('lineupInfo/mso') )
			lineup['msoId'] 	= self.check( l.find('lineupInfo/mso'), 'msoId' )
			lineup['headend'] 	= self.check( l.find('lineupInfo/headend') )
			lineup['headendId'] = self.check( l.find('lineupInfo/headend'), 'headendId' )
			lineup['device'] 	= self.check( l.find('lineupInfo/device') )
			lineup['deviceType'] = self.check( l.find('lineupInfo/device'), 'type' )
			lineup['location'] 	= self.check( l.find('lineupInfo/location') )
			lineup['type'] 		= self.check( l.find('lineupInfo/type') )

			if self.do_output_tsv:
				if not self.validate_and_write_tsv(lineup, 'lineup', 'lineupId'):
					l.clear()
					continue

			channel_inc = 0
			for lc in l.findall('channelList/channel'):

				channel = {}
				channel_inc += 1

				channelId = "{0}-{1}".format( lineupId, channel_inc )

				channel['channelId'] 	= channelId
				channel['lineupId'] 	= lineupId
				channel['updateId'] 	= updateId

				channel['prgSvcId'] 		= self.check( lc, 'prgSvcId' )
				channel['channelNumber'] 	= self.check( lc.find('channelNumber') )
				channel['effectiveDate'] 	= self.format_datetime( self.check( lc.find('channelNumber'), 'effectiveDate' ), input_format='%Y-%m-%d' )
				channel['expirationDate'] 	= self.format_datetime( self.check( lc.find('channelNumber'), 'expirationDate' ), input_format='%Y-%m-%d' )
				channel['tier'] 			= self.check( lc.find('channelNumber'), 'tier' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(channel, 'lineup_channel', 'lineupId'):
						lc.clear()
						continue

				transport_inc = 0
				for ct in lc.findall('transportIds/transportId'):

					transport = {}
					transport_inc += 1

					transportId = "{0}-{1}".format( channelId, transport_inc )

					transport['transportId'] = transportId
					transport['channelId'] 	= channelId
					transport['lineupId'] 	= lineupId
					transport['updateId'] 	= updateId

					transport['system'] 	= self.check( ct.find('system') )	
					transport['subSystem'] 	= self.check( ct.find('subSystem') )

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(transport, 'lineup_channel_transport', 'channelId'):
							ct.clear()
							continue

					for idp in ct.findall('idParts/idPart'):

						idpart = {}

						idpart['transportId'] = transportId
						idpart['lineupId'] 	= lineupId
						idpart['updateId'] 	= updateId

						idpart['type'] 	= self.check( idp.find('type') )	
						idpart['id'] 	= self.check( idp.find('id') )	

						if self.do_output_tsv:
							if not self.validate_and_write_tsv(idpart, 'lineup_channel_transport_id', 'transportId'):
								idp.clear()
								continue

						if self.do_output_jsonl:
							[ idpart.pop(key) for key in ['transportId','lineupId','updateId'] ]
							transport['ids'] = ( transport.get('ids') or [] ) + [ idpart.copy() ]

						idp.clear()

					if self.do_output_jsonl:
						[ transport.pop(key) for key in ['transportId','lineupId','channelId','updateId'] ]
						channel['transport'] = ( channel.get('transport') or [] ) + [ transport.copy() ]

					ct.clear()		


				if self.do_output_jsonl:
					[ channel.pop(key) for key in ['lineupId','updateId'] ]
					lineup['channels'] = ( lineup.get('channels') or [] ) + [ channel.copy() ]

				lc.clear()

			for lp in l.findall('lineupInfo/postalCodes/postalCode'):

				postalcode = {}

				postalcode['lineupId'] 	= lineupId
				postalcode['updateId'] 	= updateId

				postalcode['postalCode'] = self.check( lp )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(postalcode, 'lineup_postalcode', 'lineupId'):
						lp.clear()
						continue

				if self.do_output_jsonl:
					#[ postalcode.pop(key) for key in ['lineupId','updateId'] ]
					lineup['postalCodes'] = ( lineup.get('postalCodes') or [] ) + [ postalcode['postalCode'] ]

				lp.clear()

			for lm in l.findall('lineupInfo/markets/market'):

				market = {}

				market['lineupId'] 	= lineupId
				market['updateId'] 	= updateId

				market['market'] 	= self.check( lm )
				market['marketId'] 	= self.check( lm, 'marketId' )
				market['type'] 		= self.check( lm, 'type' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(market, 'lineup_market', 'lineupId'):
						lm.clear()
						continue

				if self.do_output_jsonl:
					[ market.pop(key) for key in ['lineupId','updateId'] ]
					lineup['markets'] = ( lineup.get('markets') or [] ) + [ market.copy() ]

				lm.clear()

			if self.do_output_jsonl:
				self.write_jsonl(lineup, 'lineup')

			l.clear()


# ================== ENDPOINT SPECIFIC INIT CODE BEGIN ==================
# Here insert the endpoint-specific code from schema/release/<product>/parse_.py

endpoint = 'Lineups'

entities = [
	 'lineup' 
]

tables = [
	 'lineup' 
	,'lineup_channel' 
	,'lineup_channel_transport' 
	,'lineup_channel_transport_id' 
	,'lineup_postalcode' 
	,'lineup_market' 
]

fields = {
	 'lineup': ['lineupId', 'updateId', 'updateDate', 'deleted', 'country', 'countryId', 'mso', 'msoId', 'headend', 'headendId', 'location', 'device', 'deviceType', 'type', 'MD5', 'ingestId', 'batchUpdateOrd']
	,'lineup_channel': ['channelId', 'lineupId', 'updateId', 'prgSvcId', 'channelNumber', 'effectiveDate', 'expirationDate', 'tier', 'MD5', 'ingestId']
	,'lineup_channel_transport': ['transportId', 'channelId', 'lineupId', 'updateId', 'system', 'subSystem', 'MD5', 'ingestId']
	,'lineup_channel_transport_id': ['transportId', 'lineupId', 'updateId', 'type', 'id', 'MD5', 'ingestId']
	,'lineup_postalcode': ['lineupId', 'updateId', 'postalCode', 'MD5', 'ingestId']
	,'lineup_market': ['lineupId', 'updateId', 'market', 'marketId', 'type', 'MD5', 'ingestId']

}

# =================== ENDPOINT SPECIFIC INIT CODE END ===================


gnLineupsParser = GnLineupsParser( endpoint, entities, tables, fields )
gnLineupsParser.set_config( os.environ['GXD_PDK_DIR'], os.environ['GXD_PDK_CONFIG'], files_path )
gnLineupsParser.run( xml_file )
