#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Parses a given ON ProgramAvailabilities XML file
#  Validates each record against fields configuration file
#  Writes validated records into TSV files for the endpoint
#  =============================================================================

import os
import sys

sys.path.append(os.path.join(sys.path[0],'..'))
import gxd_functions as gxd

xml_file 	= sys.argv[1]
files_path 	= sys.argv[2]

class GnProgramAvailabilitiesParser(gxd.GnEndpointParser):

	def parse(self, xml_file):

		for a in self.xml_nodes(xml_file, "on/programAvailabilities/programAvailability"):

			availability = {}

			availabilityId 	= self.check( a, 'availabilityId' )
			updateId 		= self.check( a, 'updateId' )

			availability['availabilityId'] = availabilityId
			availability['updateId'] 		= updateId

			availability['updateDate']		= self.check( a, 'updateDate' )
			availability['deleted'] 		= self.format_boolean( self.check( a, 'deleted' ))
			availability['catalogName'] 	= self.check( a.find('catalogName') )
			availability['videoQuality'] 	= self.check( a.find('videoQuality') )

			availability['availableFrom'] 	= self.check( a.find('availableFromDateTime') )
			availability['expiresAt'] 		= self.check( a.find('expiresAtDateTime') )

			# Retrieve prgSvcId and add it as part of the availability table
			prgSvcId = 0
			for ai in a.findall('ids/id'):
				if self.check( ai, 'type' ) == 'prgSvcId':
					prgSvcId = self.check( ai )
			
			availability['prgSvcId'] = prgSvcId

			if self.do_output_tsv:
				if not self.validate_and_write_tsv(availability, 'availability', 'availabilityId'):
					a.clear()
					continue

			for ai in a.findall('ids/id'):

				aid = {}

				aid['availabilityId'] 	= availabilityId
				aid['updateId'] 		= updateId

				aid['id'] 				= self.check( ai )
				aid['type'] 			= self.check( ai, 'type' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(aid, 'availability_id', 'availabilityId'):
						ai.clear()
						continue

				if self.do_output_jsonl:
					[ aid.pop(key) for key in ['availabilityId','updateId'] ]
					availability['ids'] = ( availability.get('ids') or [] ) + [ aid.copy() ]

				ai.clear()

			for au in a.findall('urls/url'):

				url = {}

				url['availabilityId'] 	= availabilityId
				url['updateId'] 		= updateId

				url['url'] 				= self.check( au )
				url['type'] 			= self.check( au, 'type' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(url, 'availability_url', 'availabilityId'):
						au.clear()
						continue

				if self.do_output_jsonl:
					[ url.pop(key) for key in ['availabilityId','updateId'] ]
					availability['urls'] = ( availability.get('urls') or [] ) + [ url.copy() ]

				au.clear()

			for av in a.findall('viewingOptions/viewingOption'):

				viewingoption = {}

				viewingoption['availabilityId'] = availabilityId
				viewingoption['updateId'] 		= updateId

				viewingoption['videoQuality'] 	= self.check( av.find('videoQuality') )
				viewingoption['license'] 		= self.check( av.find('license') )
				viewingoption['price'] 			= self.check( av.find('price') )
				viewingoption['currency'] 		= self.check( av.find('price'), 'currency' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(viewingoption, 'availability_viewingoption', 'availabilityId'):
						av.clear()
						continue

				if self.do_output_jsonl:
					[ viewingoption.pop(key) for key in ['availabilityId','updateId'] ]
					availability['viewingOptions'] = ( availability.get('viewingOptions') or [] ) + [ viewingoption.copy() ]

				av.clear()

			for ap in a.findall('providerData/datum'):

				providerdata = {}

				providerdata['availabilityId'] 	= availabilityId
				providerdata['updateId'] 		= updateId

				providerdata['key'] 			= self.check( ap.find('key') )
				providerdata['value'] 			= self.check( ap.find('value') )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(providerdata, 'availability_providerdata', 'availabilityId'):
						ap.clear()
						continue

				if self.do_output_jsonl:
					[ providerdata.pop(key) for key in ['availabilityId','updateId'] ]
					availability['providerData'] = ( availability.get('providerData') or [] ) + [ providerdata.copy() ]

				ap.clear()

			if self.do_output_jsonl:
				self.write_jsonl(availability, 'availability')

			a.clear()


# ================== ENDPOINT SPECIFIC INIT CODE BEGIN ==================
# Here insert the endpoint-specific code from schema/release/<product>/parse_.py

endpoint = 'ProgramAvailabilities'

entities = [
	 'availability' 
]

tables = [
	 'availability' 
	,'availability_id' 
	,'availability_url' 
	,'availability_viewingoption' 
	,'availability_providerdata' 
]

fields = {
	 'availability': ['availabilityId', 'updateId', 'updateDate', 'deleted', 'prgSvcId', 'catalogName', 'videoQuality', 'availableFrom', 'expiresAt', 'MD5', 'ingestId', 'batchUpdateOrd']
	,'availability_id': ['availabilityId', 'updateId', 'id', 'type', 'MD5', 'ingestId']
	,'availability_url': ['availabilityId', 'updateId', 'url', 'type', 'MD5', 'ingestId']
	,'availability_viewingoption': ['availabilityId', 'updateId', 'videoQuality', 'license', 'price', 'currency', 'MD5', 'ingestId']
	,'availability_providerdata': ['availabilityId', 'updateId', 'key', 'value', 'MD5', 'ingestId']
}


# =================== ENDPOINT SPECIFIC INIT CODE END ===================


gnProgramAvailabilitiesParser = GnProgramAvailabilitiesParser( endpoint, entities, tables, fields )
gnProgramAvailabilitiesParser.set_config( os.environ['GXD_PDK_DIR'], os.environ['GXD_PDK_CONFIG'], files_path )
gnProgramAvailabilitiesParser.run( xml_file )

