#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Parses a given ON Programs XML file
#  Validates each record against fields configuration file
#  Writes validated records into TSV files for the endpoint
#  =============================================================================

import os
import sys

sys.path.append(os.path.join(sys.path[0],'..'))
import gxd_functions as gxd

xml_file 	= sys.argv[1]
files_path 	= sys.argv[2]

class GnProgramsParser(gxd.GnEndpointParser):

	def parse(self, xml_file):

		for p in self.xml_nodes(xml_file, "on/programs/program"):

			program = {}

			TMSId 		= self.check( p, 'TMSId' )
			updateId 	= self.check( p, 'updateId' )
			updateDate	= self.check( p, 'updateDate' )

			program['TMSId'] 	 	= TMSId
			program['updateId'] 	= updateId
			program['updateDate']	= updateDate

			program['rootId'] 		= self.check( p, 'rootId' )
			program['seasonId'] 	= self.check( p, 'seasonId' )	# Episode parent season Id
			program['seriesId'] 	= self.check( p, 'seriesId' )

			program['versionId']	= self.check( p, 'versionId' )
			program['connectorId']	= self.check( p, 'connectorId' )

			is_program_deleted 		= self.format_boolean( self.check( p, 'deleted' ))
			program['deleted'] 		= is_program_deleted
			is_program_inactive		= self.format_boolean( self.check( p, 'inactive' ))
			program['inactive'] 	= is_program_inactive

			program['title_lang'] 	= self.check( p.find('titles'), 'lang' )
			program['desc_lang'] 	= self.check( p.find('descriptions'), 'lang' )

			# Retrieve title subType of type='full' and add it as part of the program table
			for pt in p.findall('titles/title'):
				if self.check( pt, 'type' ) == 'full':
					program['title_subType'] = self.check( pt, 'subType' )

			program['runTime'] 		= self.check( p.find('runTime') )
			program['progType'] 	= self.check( p.find('progType') )
			program['subType'] 		= self.check( p.find('subType') )

			if self.do_parse_all:
				program['holiday'] 		= self.check( p.find('holiday') )
				program['holidayId']  	= self.check( p.find('holiday'), 'holidayId' )

				program['origAirDate'] 	= self.check( p.find('origAirDate') )

				program['colorCode'] 	= self.check( p.find('colorCode') )
				program['versionLabel'] = self.check( p.find('versionLabel') )
				program['animation'] 	= self.check( p.find('animation') )
				program['origAudioLang'] = self.check( p.find('origAudioLang') )
				program['sourceType'] 	= self.check( p.find('sourceType') )

				program['seriesPremiere'] = self.check( p.find('seriesPremiere') )
				program['seriesFinale']	= self.check( p.find('seriesFinale') )

				program['showcard'] 	= self.format_boolean( self.check( p.find('showcard') ))
				program['officialURL']  = self.check( p.find('officialURL') )
				program['partNumber']  	= self.check( p.find('partNumber') )
				program['totalParts']  	= self.check( p.find('totalParts') )
				program['duration'] 	= self.check( p.find('duration') )

				program['origNetwork'] 			= self.check( p.find('originalNetwork') )
				program['origNetwork_prgSvcId'] = self.check( p.find('originalNetwork'), 'prgSvcId' )
				program['lastOrigNetwork'] 		= self.check( p.find('lastOriginalNetwork') )
				program['lastorigNetwork_prgSvcId']= self.check( p.find('lastOriginalNetwork'), 'prgSvcId' )

			if self.do_output_tsv:
				if not self.validate_and_write_tsv(program, 'program', 'TMSId'):
					p.clear()
					continue


			for s in p.findall('seasons/season'):

				# We're treating seasons as top level entities in the DB schema even though API provides season data embedded in program XML
				# Season data is delivered on a series (SH) record, however different series records can deliver different parts of the same season data with the same seasonId
				# Season object has a seasonId (which is really a rootId in GN terms), but it does not have an id differentiating between different season language variants
				# So we will construct a composite key for the Season object, consisting of seasonId (equivalent of a rootId) and a parent SH program TMSId

				seasonId 		= self.check( s, 'seasonId' )				# season object (root) id			
				seasonTMSId 	= "{}-{}".format( seasonId, TMSId )		# season object composite primary key

				season = {}

				season['updateId'] 		= updateId
				season['updateDate']	= updateDate
				season['seasonTMSId'] 	= seasonTMSId
				season['seasonId'] 		= seasonId
				season['deleted'] 		= is_program_deleted
				season['inactive'] 		= is_program_inactive

				season['seasonNumber'] 	= self.check( s, 'seasonNumber' )

				if self.do_parse_all:
					season['seasonYear'] 		= self.check( s, 'seasonYear' )
					season['seasonPremiere']	= self.check( s.find('seasonPremiere') )
					season['seasonFinale']  	= self.check( s.find('seasonFinale') )

					season['origNetwork'] 			= self.check( s.find('originalNetwork') )
					season['origNetwork_prgSvcId']	= self.check( s.find('originalNetwork'), 'prgSvcId' )
					
					season['totalSeasonEpisodes']	= self.check( s.find('totalSeasonEpisodes') )

					season['showcard'] 	  		= self.format_boolean( self.check( s.find('showcard') ))
					season['officialURL']  		= self.check( s.find('officialURL') )
					season['title_lang'] 		= self.check( s.find('titles'), 'lang' )
					season['desc_lang']  		= self.check( s.find('descriptions'), 'lang' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(season, 'season', 'seasonTMSId'):
						s.clear()
						continue

				# Link between (SH) Program and its Seasons
				program_season = {}
				program_season['updateId'] 		= updateId
				program_season['TMSId'] 		= TMSId
				program_season['seasonTMSId'] 	= seasonTMSId

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(program_season, 'program_season', 'TMSId') and not self.rdb_skeleton:
						s.clear()
						continue

				if self.do_output_jsonl:
					#[ program_season.pop(key) for key in ['TMSId','updateId'] ]
					program['seasons'] = ( program.get('seasons') or [] ) + [ program_season['seasonTMSId'] ]


				for st in s.findall('titles/title'):

					title = {}

					title['updateId'] 		= updateId
					title['seasonTMSId'] 	= seasonTMSId

					title['title']		= self.check( st )
					title['size'] 		= self.check( st, 'size' )
					title['type'] 		= self.check( st, 'type' )
					title['lang'] 		= self.check( st, 'lang' )
					title['subType']	= self.check( st, 'subType' )

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(title, 'season_title', 'seasonTMSId'):
							st.clear()
							continue

					if self.do_output_jsonl:
						[ title.pop(key) for key in ['seasonTMSId','updateId'] ]
						season['titles'] = ( season.get('titles') or [] ) + [ title.copy() ]

					st.clear()

				for sd in s.findall('descriptions/desc'):

					description = {}

					description['updateId'] 	= updateId
					description['seasonTMSId'] 	= seasonTMSId

					description['description'] 	= self.check( sd )
					description['size'] 		= self.check( sd, 'size' )
					description['type'] 		= self.check( sd, 'type' )
					description['lang'] 		= self.check( sd, 'lang' )

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(description, 'season_description', 'seasonTMSId'):
							sd.clear()
							continue

					if self.do_output_jsonl:
						[ description.pop(key) for key in ['seasonTMSId','updateId'] ]
						season['descriptions'] = ( season.get('descriptions') or [] ) + [ description.copy() ]

					sd.clear()

				for sc in s.findall('cast/member'):

					cast = {}

					cast['updateId'] 		= updateId
					cast['seasonTMSId'] 	= seasonTMSId

					cast['ord'] 			= self.check( sc, 'ord' )
					cast['personId'] 		= self.check( sc, 'personId' )
					cast['characterName'] 	= self.check( sc.find('characterName') )
					cast['role'] 			= self.check( sc.find('role') )
					cast['roleId'] 			= self.check( sc.find('role'), 'roleId' )
					cast['nameId'] 			= self.check( sc.find('name'), 'nameId' )
					cast['departed'] 	  	= self.format_boolean( self.check( sc.find('departed') ))

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(cast, 'season_cast', 'seasonTMSId'):
							sc.clear()
							continue

					if self.do_output_jsonl:
						[ cast.pop(key) for key in ['seasonTMSId','updateId'] ]
						season['cast'] = ( season.get('cast') or [] ) + [ cast.copy() ]

					sc.clear()

				for sc in s.findall('crew/member'):

					crew = {}

					crew['updateId'] 		= updateId
					crew['seasonTMSId'] 	= seasonTMSId

					crew['ord'] 			= self.check( sc, 'ord' )
					crew['personId'] 		= self.check( sc, 'personId' )
					crew['role'] 			= self.check( sc.find('role') )
					crew['roleId'] 			= self.check( sc.find('role'), 'roleId' )
					crew['nameId'] 			= self.check( sc.find('name'), 'nameId' )

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(crew, 'season_crew', 'seasonTMSId'):
							sc.clear()
							continue

					if self.do_output_jsonl:
						[ crew.pop(key) for key in ['seasonTMSId','updateId'] ]
						season['crew'] = ( season.get('crew') or [] ) + [ crew.copy() ]

					sc.clear()

				asset_inc = 0
				for sa in s.findall('assets/asset'):

					asset = {}

					assetId = self.check( sa, 'assetId' )
					asset_inc += 1
					seasonAssetId = "{0}-{1}".format( seasonTMSId, asset_inc )

					asset['seasonAssetId'] 	= seasonAssetId
					asset['updateId'] 		= updateId
					asset['seasonTMSId'] 	= seasonTMSId
					asset['assetId'] 		= assetId

					asset['type'] 			= self.check( sa, 'type' )
					asset['expiredDate'] 	= self.check( sa, 'expiredDate' )
					asset['lastModified'] 	= self.check( sa, 'lastModified' )
					asset['width'] 			= self.check( sa, 'width' )
					asset['height'] 		= self.check( sa, 'height' )
					asset['category'] 		= self.check( sa, 'category' )
					asset['ratio'] 			= self.check( sa, 'ratio' )
					asset['tier'] 			= self.check( sa, 'tier' )
					asset['provider'] 		= self.check( sa, 'provider' )
					asset['seasonId'] 		= self.check( sa, 'seasonId' )
					asset['primary'] 		= self.format_boolean( self.check( sa, 'primary' ))
					asset['deleted'] 		= self.format_boolean( self.check( sa, 'deleted' ))			
					asset['URI'] 			= self.check( sa.find('URI') )
					asset['caption'] 		= self.check( sa.find('caption') )
					asset['caption_lang'] 	= self.check( sa.find('caption'), 'lang' )

					asset['restrictedProvider'] = self.format_boolean( self.check( sa, 'restrictedProvider' ))

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(asset, 'season_asset', 'seasonAssetId'):
							sa.clear()
							continue

					for sa_identifier in sa.findall('identifiers/identifier'):

						# The XML schema here somewhat deviates from typical convention (items/item):
						# The actual elements (<id> tag) are nested under <identifier> tag which carries the type attribute

						sa_identifier_type = self.check( sa_identifier, 'type' )

						for sa_id in sa_identifier.findall('id'):

							asset_id = {}

							asset_id['seasonAssetId'] 	= seasonAssetId
							asset_id['updateId'] 		= updateId
							asset_id['seasonTMSId'] 	= seasonTMSId
							asset_id['type'] 			= sa_identifier_type
							asset_id['id'] 				= self.check( sa_id )

							if self.do_output_tsv:
								if not self.validate_and_write_tsv(asset_id, 'season_asset_id', 'seasonAssetId'):
									sa_id.clear()
									continue

							if self.do_output_jsonl:
								[ asset_id.pop(key) for key in ['seasonAssetId','seasonTMSId','updateId'] ]
								asset['identifiers'] = ( asset.get('identifiers') or [] ) + [ asset_id.copy() ]

							sa_id.clear()

						sa_identifier.clear()

					if self.do_output_jsonl:
						[ asset.pop(key) for key in ['seasonAssetId','seasonTMSId','updateId'] ]
						season['assets'] = ( season.get('assets') or [] ) + [ asset.copy() ]

					sa.clear()

				if self.do_output_jsonl:
					self.write_jsonl(season, 'season')

				s.clear()	

			# Back to program child tables
			for pt in p.findall('titles/title'):

				title = {}

				title['updateId'] 	= updateId
				title['TMSId'] 		= TMSId

				title['title'] 		= self.check( pt )
				title['size'] 		= self.check( pt, 'size' )
				title['type'] 		= self.check( pt, 'type' )
				title['lang'] 		= self.check( pt, 'lang' )
				title['subType'] 	= self.check( pt, 'subType' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(title, 'program_title', 'TMSId'):
						pt.clear()
						continue

				if self.do_output_jsonl:
					[ title.pop(key) for key in ['TMSId','updateId'] ]
					program['titles'] = ( program.get('titles') or [] ) + [ title.copy() ]

				pt.clear()

			for pd in p.findall('descriptions/desc'):

				description = {}

				description['updateId'] 	= updateId
				description['TMSId'] 		= TMSId

				description['description']	= self.check( pd )
				description['size']			= self.check( pd, 'size' )
				description['type']			= self.check( pd, 'type' )
				description['lang']			= self.check( pd, 'lang' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(description, 'program_description', 'TMSId'):
						pd.clear()
						continue

				if self.do_output_jsonl:
					[ description.pop(key) for key in ['TMSId','updateId'] ]
					program['descriptions'] = ( program.get('descriptions') or [] ) + [ description.copy() ]

				pd.clear()

			for pc in p.findall('cast/member'):

				cast = {}

				cast['updateId'] 	= updateId
				cast['TMSId'] 		= TMSId

				cast['ord'] 			= self.check( pc, 'ord' )
				cast['personId'] 		= self.check( pc, 'personId' )
				cast['characterName'] 	= self.check( pc.find('characterName') )
				cast['role'] 			= self.check( pc.find('role') )
				cast['roleId'] 			= self.check( pc.find('role'), 'roleId' )
				cast['nameId'] 			= self.check( pc.find('name'), 'nameId' )
				cast['departed'] 	  	= self.format_boolean( self.check( pc.find('departed') ))

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(cast, 'program_cast', 'TMSId'):
						pc.clear()
						continue

				if self.do_output_jsonl:
					[ cast.pop(key) for key in ['TMSId','updateId'] ]
					program['cast'] = ( program.get('cast') or [] ) + [ cast.copy() ]

				pc.clear()

			for pc in p.findall('crew/member'):

				crew = {}

				crew['updateId'] 	= updateId
				crew['TMSId'] 		= TMSId

				crew['ord'] 			= self.check( pc, 'ord' )
				crew['personId'] 		= self.check( pc, 'personId' )
				crew['role'] 			= self.check( pc.find('role') )
				crew['roleId'] 			= self.check( pc.find('role'), 'roleId' )
				crew['nameId'] 			= self.check( pc.find('name'), 'nameId' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(crew, 'program_crew', 'TMSId'):
						pc.clear()
						continue

				if self.do_output_jsonl:
					[ crew.pop(key) for key in ['TMSId','updateId'] ]
					program['crew'] = ( program.get('crew') or [] ) + [ crew.copy() ]

				pc.clear()

			for pc in p.findall('countries/country'):

				country = {}

				country['updateId'] 	= updateId
				country['TMSId'] 		= TMSId

				country['country'] 		= self.check( pc )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(country, 'program_country', 'TMSId'):
						pc.clear()
						continue

				if self.do_output_jsonl:
					#[ country.pop(key) for key in ['TMSId','updateId'] ]
					program['countries'] = ( program.get('countries') or [] ) + [ country['country'] ]

				pc.clear()

			for pa in p.findall('awards/award'):

				award = {}

				award['updateId'] 	= updateId
				award['TMSId'] 		= TMSId

				award['won'] 				= self.format_boolean( self.check( pa, 'won' ))
				award['award'] 				= self.check( pa.find('name') )
				award['awardId'] 			= self.check( pa.find('name'), 'awardId' )
				award['category'] 			= self.check( pa.find('category') )
				award['awardCatId'] 		= self.check( pa.find('category'), 'awardCatId' )
				award['recipient'] 			= self.check( pa.find('recipient') )
				award['recipientNameId'] 	= self.check( pa.find('recipient'), 'nameId' )
				award['year'] 				= self.check( pa.find('year') )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(award, 'program_award', 'TMSId'):
						pa.clear()
						continue

				if self.do_output_jsonl:
					[ award.pop(key) for key in ['TMSId','updateId'] ]
					program['awards'] = ( program.get('awards') or [] ) + [ award.copy() ]

				pa.clear()

			genre_inc = 0
			for pg in p.findall('genres/genre'):

				genre = {}
				genre_inc += 1

				genre['updateId'] 	= updateId
				genre['TMSId'] 		= TMSId

				genre['genreOrd']	= genre_inc
				genre['genre']		= self.check( pg )
				genre['genreId'] 	= self.check( pg, 'genreId' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(genre, 'program_genre', 'TMSId'):
						pg.clear()
						continue

				if self.do_output_jsonl:
					[ genre.pop(key) for key in ['TMSId','updateId'] ]
					program['genres'] = ( program.get('genres') or [] ) + [ genre.copy() ]

				pg.clear()


			for poal in p.findall('audioOriginalLangs/audioOriginalLang'):

				oal = {}

				oal['updateId'] 	= updateId
				oal['TMSId'] 		= TMSId

				oal['ord'] 		= self.check( poal, 'ord' )
				oal['lang']		= self.check( poal )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(oal, 'program_origaudiolang', 'TMSId'):
						poal.clear()
						continue

				if self.do_output_jsonl:
					[ oal.pop(key) for key in ['TMSId','updateId'] ]
					program['origAudioLangs'] = ( program.get('origAudioLangs') or [] ) + [ oal.copy() ]

				poal.clear()


			for pr in p.findall('ratings/rating'):

				rating = {}

				rating['updateId'] 		= updateId
				rating['TMSId'] 		= TMSId

				rating['warning']		= self.check( pr.find('warning') )
				rating['exempt'] 		= self.format_boolean( self.check( pr, 'exempt' ))
				rating['rating_code'] 	= self.check( pr, 'code' )
				rating['ratingBodyId'] 	= self.check( pr, 'ratingBodyId' )
				rating['ratingsBody'] 	= self.check( pr, 'ratingsBody' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(rating, 'program_rating', 'TMSId'):
						pr.clear()
						continue

				if self.do_output_jsonl:
					[ rating.pop(key) for key in ['TMSId','updateId'] ]
					program['ratings'] = ( program.get('ratings') or [] ) + [ rating.copy() ]

				pr.clear()

			for pqr in p.findall('ratings/qualityRating'):

				qualityrating = {}

				qualityrating['updateId'] 	= updateId
				qualityrating['TMSId'] 		= TMSId

				qualityrating['ratingsBody'] 	= self.check( pqr, 'ratingsBody' )
				qualityrating['numVotes'] 		= self.check( pqr, 'numVotes' )
				qualityrating['value'] 			= self.check( pqr, 'value' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(qualityrating, 'program_qualityrating', 'TMSId'):
						pqr.clear()
						continue

				if self.do_output_jsonl:
					[ qualityrating.pop(key) for key in ['TMSId','updateId'] ]
					program['quality_ratings'] = ( program.get('quality_ratings') or [] ) + [ qualityrating.copy() ]

				pqr.clear()

			for pra in p.findall('ratings/advisories/advisory'):

				ratingadvisory = {}

				ratingadvisory['updateId'] 		= updateId
				ratingadvisory['TMSId'] 		= TMSId

				ratingadvisory['advisory']		= self.check( pra )
				ratingadvisory['ratingsBody'] 	= self.check( pra, 'ratingsBody' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(ratingadvisory, 'program_ratingadvisory', 'TMSId'):
						pra.clear()
						continue

				if self.do_output_jsonl:
					[ ratingadvisory.pop(key) for key in ['TMSId','updateId'] ]
					program['rating_advisories'] = ( program.get('rating_advisories') or [] ) + [ ratingadvisory.copy() ]

				pra.clear()

			pmi = p.find('movieInfo')
			if pmi is not None:

				movieinfo = [    ('yearOfRelease', 		'yearOfRelease')
								,('officialURL', 		'officialURL')
								,('productionCompany', 	'productionCompanies/name')
								,('soundMix', 			'soundMixes/soundMix')
								,('pictureFormat', 		'pictureFormats/pictureFormat')		]

				for key, xpath in movieinfo:
					for pmi_value in pmi.findall(xpath):

						mvinfo = {}

						mvinfo['updateId'] 	= updateId
						mvinfo['TMSId'] 		= TMSId

						mvinfo['key']		= key
						mvinfo['value'] 		= self.check( pmi_value )

						if self.do_output_tsv:
							if not self.validate_and_write_tsv(mvinfo, 'program_mvinfo', 'TMSId'):
								pmi_value.clear()
								continue

						if self.do_output_jsonl:
							[ mvinfo.pop(key) for key in ['TMSId','updateId'] ]
							program['movieInfo'] = ( program.get('movieInfo') or [] ) + [ mvinfo.copy() ]

						pmi_value.clear()

				for pmit in p.findall('movieInfo/trailers/trailer'):

					movietrailer = {}

					movietrailer['updateId'] 	= updateId
					movietrailer['TMSId'] 		= TMSId

					movietrailer['format']		= self.check( pmit.find('format') )
					movietrailer['URL'] 		= self.check( pmit.find('URL') )

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(movietrailer, 'program_mvtrailer', 'TMSId'):
							pmit.clear()
							continue

					if self.do_output_jsonl:
						[ movietrailer.pop(key) for key in ['TMSId','updateId'] ]
						program['movieTrailers'] = ( program.get('movieTrailers') or [] ) + [ movietrailer.copy() ]

					pmit.clear()

				pmi.clear()

			episodeinfo_id = 0
			for pei in p.findall('episodeInfo'):

				episodeinfo = {}

				episodeinfo_id += 1
				episodeInfoId = "{0}-{1}".format( TMSId, str(episodeinfo_id) )

				episodeinfo['episodeInfoId'] 	= episodeInfoId
				episodeinfo['TMSId'] 			= TMSId
				episodeinfo['updateId'] 		= updateId

				title 							= pei.find('title')
				episodeinfo['title']			= self.check( title )
				episodeinfo['size'] 			= self.check( title, 'size' )
				episodeinfo['type'] 			= self.check( title, 'type' )
				episodeinfo['subType'] 			= self.check( title, 'subType' )
				episodeinfo['lang'] 			= self.check( title, 'lang' )

				episodeinfo['season'] 			= self.check( pei, 'season' )
				episodeinfo['number'] 			= self.check( pei, 'number' )
				episodeinfo['episodes'] 		= self.check( pei, 'episodes' )
				episodeinfo['numInSeries'] 		= self.check( pei, 'numInSeries' )
				episodeinfo['seasonYear'] 		= self.check( pei, 'seasonYear' )
				episodeinfo['synNum']			= self.check( pei.find('synNum') )
				episodeinfo['altSynNum'] 		= self.check( pei.find('altSynNum') )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(episodeinfo, 'program_epinfo', 'TMSId'):
						pei.clear()
						if title is not None:
							title.clear()
						continue

				for peia in pei.findall('alternate'):

					alternate = {}

					alternate['episodeInfoId'] 	= episodeInfoId
					alternate['TMSId'] 			= TMSId
					alternate['updateId'] 		= updateId

					alternate['countrywide'] = self.format_boolean( self.check( peia, 'countrywide' ))
					alternate['country'] 	= self.check( peia, 'country' )
					alternate['prgSvcId'] 	= self.check( peia, 'prgSvcId' )
					alternate['season'] 	= self.check( peia, 'season' )
					alternate['episode'] 	= self.check( peia, 'episode' )

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(alternate, 'program_epinfo_alt', 'TMSId'):
							peia.clear()
							continue

					if self.do_output_jsonl:
						[ alternate.pop(key) for key in ['episodeInfoId','TMSId','updateId'] ]
						episodeinfo['alternate'] = ( episodeinfo.get('alternate') or [] ) + [ alternate.copy() ]

					peia.clear()

				if self.do_output_jsonl:
					[ episodeinfo.pop(key) for key in ['episodeInfoId','TMSId','updateId'] ]
					program['episodeInfo'] = ( program.get('episodeInfo') or [] ) + [ episodeinfo.copy() ]

				pei.clear()
				if title is not None:
					title.clear()

			for psi in p.findall('sportsInfo'):

				sportsinfo = {}

				sportsinfo['updateId'] 	= updateId
				sportsinfo['TMSId'] 	= TMSId

				sportsinfo['gameDate']			= self.check( psi.find('gameDate') )
				sportsinfo['gameTime']			= self.check( psi.find('gameTime') )
				sportsinfo['gameTimeZone']		= self.check( psi.find('gameTimeZone') )
				sportsinfo['venue']				= self.check( psi.find('venue') )
				sportsinfo['venueBrandId'] 		= self.check( psi.find('venue'), 'venueBrandId' )
				sportsinfo['season']			= self.check( psi.find('season') )
				sportsinfo['season_type'] 		= self.check( psi.find('season'), 'type' )
				sportsinfo['playoffRound']		= self.check( psi.find('playoffRound') )
				sportsinfo['playoffRound_gameNum'] 	= self.check( psi.find('playoffRound'), 'gameNumber' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(sportsinfo, 'program_spinfo', 'TMSId'):
						psi.clear()
						continue


				for psit in psi.findall('team'):

					sportsinfoteam = {}

					sportsinfoteam['updateId'] 		= updateId
					sportsinfoteam['TMSId'] 		= TMSId

					sportsinfoteam['team']			= self.check( psit )
					sportsinfoteam['teamBrandId'] 	= self.check( psit, 'teamBrandId' )
					sportsinfoteam['isHome'] 		= self.format_boolean( self.check( psit, 'isHome' ))

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(sportsinfoteam, 'program_spteam', 'TMSId'):
							psit.clear()
							continue

					if self.do_output_jsonl:
						[ sportsinfoteam.pop(key) for key in ['TMSId','updateId'] ]
						sportsinfo['team'] = ( sportsinfo.get('team') or [] ) + [ sportsinfoteam.copy() ]

					psit.clear()

				if self.do_output_jsonl:
					[ sportsinfo.pop(key) for key in ['TMSId','updateId'] ]
					program['sportsInfo'] = ( program.get('sportsInfo') or [] ) + [ sportsinfo.copy() ]

				psi.clear()


			for pse in p.findall('sportsData/sportsEvents/sportsEvent'):

				sportsevent = {}

				sportsevent['updateId'] 	= updateId
				sportsevent['TMSId'] 		= TMSId

				sportsevent['eventId']		= self.check( pse.find('id') )
				sportsevent['type']			= self.check( pse.find('type') )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(sportsevent, 'program_spevent', 'TMSId'):
						pse.clear()
						continue

				if self.do_output_jsonl:
					[ sportsevent.pop(key) for key in ['TMSId','updateId'] ]
					program['sportsEvents'] = ( program.get('sportsEvents') or [] ) + [ sportsevent.copy() ]

				pse.clear()

# onSportsData

			spdata_inc = 0
			for psd in p.findall('onSportsData/sportsDatum'):

				spdata = {}

				spdata_inc += 1
				sportsDataId = "{0}-{1}".format( TMSId, spdata_inc )

				spdata['sportsDataId'] 	= sportsDataId
				spdata['TMSId'] 		= TMSId
				spdata['updateId'] 		= updateId

				spdata['homeParticipantGId'] = self.check( psd.find('homeParticipantGId') )
				spdata['awayParticipantGId'] = self.check( psd.find('awayParticipantGId') )
				spdata['participantType'] 	= self.check( psd.find('participantType') )

				spdata['sportGId'] 		= self.check( psd.find('sportGId') )
				spdata['venueGId'] 		= self.check( psd.find('venueGId') )
				spdata['gender'] 		= self.check( psd.find('gender') )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(spdata, 'program_spdata', 'TMSId'):
						psd.clear()
						continue

				for psde in psd.findall('sportsEvents/sportsEvent'):

					spdata_event = {}

					spdata_event['sportsDataId'] 	= sportsDataId
					spdata_event['TMSId'] 			= TMSId
					spdata_event['updateId'] 		= updateId

					spdata_event['sportsEventGId'] 	= self.check( psde, 'sportsEventGId' )
					spdata_event['type'] 			= self.check( psde, 'type' )

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(spdata_event, 'program_spdata_event', 'sportsDataId'):
							psde.clear()
							continue

					if self.do_output_jsonl:
						[ spdata_event.pop(key) for key in ['sportsDataId','TMSId','updateId'] ]
						spdata['events'] = ( spdata.get('events') or [] ) + [ spdata_event.copy() ]

					psde.clear()

				if self.do_output_jsonl:
					[ spdata.pop(key) for key in ['sportsDataId','TMSId','updateId'] ]
					program['sportsData'] = ( program.get('sportsData') or [] ) + [ spdata.copy() ]

				psd.clear()

# onSportsData


			asset_inc = 0
			for pa in p.findall('assets/asset'):

				asset = {}

				assetId = self.check( pa, 'assetId' )
				asset_inc += 1
				programAssetId = "{0}-{1}".format( TMSId, asset_inc )

				asset['programAssetId'] = programAssetId
				asset['TMSId'] 			= TMSId
				asset['updateId'] 		= updateId
				asset['assetId'] 		= assetId

				asset['type'] 			= self.check( pa, 'type' )
				asset['expiredDate'] 	= self.check( pa, 'expiredDate' )
				asset['lastModified'] 	= self.check( pa, 'lastModified' )
				asset['width'] 			= self.check( pa, 'width' )
				asset['height'] 		= self.check( pa, 'height' )
				asset['category'] 		= self.check( pa, 'category' )
				asset['ratio'] 			= self.check( pa, 'ratio' )
				asset['tier'] 			= self.check( pa, 'tier' )
				asset['provider'] 		= self.check( pa, 'provider' )
				asset['primary'] 		= self.format_boolean( self.check( pa, 'primary' ))
				asset['deleted'] 		= self.format_boolean( self.check( pa, 'deleted' ))			
				asset['URI'] 			= self.check( pa.find('URI') )
				asset['caption'] 		= self.check( pa.find('caption') )
				asset['caption_lang'] 	= self.check( pa.find('caption'), 'lang' )

				asset['restrictedProvider'] = self.format_boolean( self.check( pa, 'restrictedProvider' ))

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(asset, 'program_asset', 'programAssetId'):
						pa.clear()
						continue

				for pa_identifier in pa.findall('identifiers/identifier'):

					# The XML schema here somewhat deviates from typical convention (items/item):
					# The actual elements (<id> tag) are nested under <identifier> tag which carries the type attribute

					pa_identifier_type = self.check( pa_identifier, 'type' )

					for pa_id in pa_identifier.findall('id'):

						asset_id = {}

						asset_id['programAssetId'] 	= programAssetId
						asset_id['TMSId'] 			= TMSId
						asset_id['updateId'] 		= updateId
						asset_id['type'] 			= pa_identifier_type
						asset_id['id'] 				= self.check( pa_id )

						if self.do_output_tsv:
							if not self.validate_and_write_tsv(asset_id, 'program_asset_id', 'programAssetId'):
								pa_id.clear()
								continue

						if self.do_output_jsonl:
							[ asset_id.pop(key) for key in ['programAssetId','TMSId','updateId'] ]
							asset['identifiers'] = ( asset.get('identifiers') or [] ) + [ asset_id.copy() ]

						pa_id.clear()

					pa_identifier.clear()

				if self.do_output_jsonl:
					[ asset.pop(key) for key in ['programAssetId','TMSId','updateId'] ]
					program['assets'] = ( program.get('assets') or [] ) + [ asset.copy() ]

				pa.clear()

			release_id = 0
			for pr in p.findall('releases/release'):

				release = {}

				# Since program releases have grandchild objects (attributes and distributors), we must create a unique primary key for program releases
				# Fields in program releases are not striclty required and/or may contain duplicates
				# We will create a composite primary key consisting of program TMSId and incrementing integer
				# Alternative would be to build a composite key from several non-null fields such as country/medium/date and de-dupe program_release* data using MD5

				release_id += 1
				releaseId = "{0}-{1}".format( TMSId, str(release_id))

				country = self.check( pr.find('country') )
				medium 	= self.check( pr.find('medium') )
				date 	= self.check( pr.find('date') )

				release['releaseId']	= releaseId
				release['TMSId'] 		= TMSId
				release['updateId'] 	= updateId

				release['country']		= country
				release['medium']		= medium
				release['date']			= date

				release['type']			= self.check( pr.find('type') )
				release['editNotes']	= self.check( pr.find('editNotes') )
				release['prgSvcId']		= self.check( pr.find('prgSvcId') )
				release['seasonYear']	= self.check( pr.find('seasonYear') )
				release['seasonName']	= self.check( pr.find('seasonName') )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(release, 'program_release', 'TMSId'):
						pr.clear()
						continue

				for pra in pr.findall('attributes/attribute'):

					attribute = {}

					attribute['releaseId'] 	= releaseId
					attribute['TMSId'] 		= TMSId
					attribute['updateId'] 	= updateId

					attribute['attribute']		= self.check( pra )
					attribute['attribute_type']	= self.check( pra, 'type' )

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(attribute, 'program_release_attribute', 'TMSId'):
							pra.clear()
							continue

					if self.do_output_jsonl:
						[ attribute.pop(key) for key in ['releaseId','TMSId','updateId'] ]
						release['attributes'] = ( release.get('attributes') or [] ) + [ attribute.copy() ]

					pra.clear()

				for prd in pr.findall('distributors/name'):

					distributor = {}

					distributor['releaseId'] 	= releaseId
					distributor['TMSId'] 		= TMSId
					distributor['updateId'] 	= updateId

					distributor['name']		= self.check( prd )

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(distributor, 'program_release_distributor', 'TMSId'):
							prd.clear()
							continue

					if self.do_output_jsonl:
						#[ distributor.pop(key) for key in ['releaseId','TMSId','updateId'] ]
						release['distributors'] = ( release.get('distributors') or [] ) + [ distributor['name'] ]

					prd.clear()

				if self.do_output_jsonl:
					[ release.pop(key) for key in ['releaseId','TMSId','updateId'] ]
					program['releases'] = ( program.get('releases') or [] ) + [ release.copy() ]

				pr.clear()

			for pr in p.findall('relationships/relationship'):

				rel = {}

				rel['TMSId'] 			= TMSId
				rel['updateId'] 		= updateId

				rel['rootId'] 			= self.check( pr.find('rootId') )
				rel['organizationId'] 	= self.check( pr.find('organizationId') )
				rel['type'] 			= self.check( pr.find('type') )
				rel['association'] 		= self.check( pr.find('association') )
				rel['personId'] 		= self.check( pr.find('personId') )
				rel['personNameId'] 	= self.check( pr.find('personNameId') )
				rel['label']	 		= self.check( pr.find('label') )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(rel, 'program_relationship', 'TMSId'):
						pr.clear()
						continue

				if self.do_output_jsonl:
					[ rel.pop(key) for key in ['TMSId','updateId'] ]
					program['relationships'] = ( program.get('relationships') or [] ) + [ rel.copy() ]

				pr.clear()

			for pxl in p.findall('externalLinks/externalLink'):

				extlink = {}

				extlink['updateId'] 	= updateId
				extlink['TMSId'] 		= TMSId

				extlink['vendor'] 	= self.check( pxl, 'vendor' )
				extlink['vendorId'] = self.check( pxl, 'vendorId' )
				extlink['id'] 		= self.check( pxl, 'id' )
				extlink['url'] 		= self.check( pxl, 'url' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(extlink, 'program_externallink', 'TMSId'):
						pxl.clear()
						continue

				pxl.clear()

			for pvd in p.findall('videoDescriptors/videoDescriptor'):

				descriptor = {}

				descriptor['updateId'] 		= updateId
				descriptor['TMSId'] 		= TMSId

				descriptor['ord'] 			= self.check( pvd, 'ord' )
				descriptor['descriptorId'] 	= self.check( pvd, 'id' )
				descriptor['weight'] 		= self.check( pvd, 'weight' )
				descriptor['type'] 			= self.check( pvd, 'type' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(descriptor, 'program_descriptor', 'TMSId'):
						pvd.clear()
						continue

				if self.do_output_jsonl:
					[ descriptor.pop(key) for key in ['TMSId','updateId'] ]
					program['videoDescriptors'] = ( program.get('videoDescriptors') or [] ) + [ descriptor.copy() ]

				pvd.clear()

			if self.do_output_jsonl:
				self.write_jsonl(program, 'program')

			p.clear()


# ================== ENDPOINT SPECIFIC INIT CODE BEGIN ==================
# Here insert the endpoint-specific code from schema/release/<product>/parse_.py


endpoint = 'Programs'

entities = [
	 'program' 
	,'season' 
]

tables = [
	 'program' 
	,'program_season' 
	,'program_title' 
	,'program_description' 
	,'program_cast' 
	,'program_crew' 
	,'program_country' 
	,'program_award' 
	,'program_genre' 
	,'program_rating' 
	,'program_qualityrating' 
	,'program_ratingadvisory' 
	,'program_mvinfo' 
	,'program_mvtrailer' 
	,'program_epinfo' 
	,'program_epinfo_alt' 
	,'program_spinfo' 
	,'program_spteam' 
	,'program_spevent' 	
	,'program_spdata' 
	,'program_spdata_event' 
	,'program_asset' 
	,'program_asset_id' 
	,'program_release' 
	,'program_release_attribute' 
	,'program_release_distributor' 
	,'program_relationship' 
	,'program_externallink' 
	,'program_descriptor' 
	,'program_origaudiolang' 
	,'season' 
	,'season_title' 
	,'season_description' 
	,'season_cast' 
	,'season_crew' 
	,'season_asset' 
	,'season_asset_id' 

]

fields = {
	 'program': ['TMSId', 'updateId', 'updateDate', 'deleted', 'inactive', 'rootId', 'versionId', 'connectorId', 'seasonId', 'seriesId', 'title_lang', 'desc_lang', 'title_subType', 'runTime', 'progType', 'subType', 'holiday', 'holidayId', 'origAirDate', 'colorCode', 'versionLabel', 'animation', 'origAudioLang', 'sourceType', 'origNetwork', 'origNetwork_prgSvcId', 'lastOrigNetwork', 'lastOrigNetwork_prgSvcId', 'seriesPremiere', 'seriesFinale', 'showcard', 'officialURL', 'partNumber', 'totalParts', 'duration', 'MD5', 'ingestId', 'batchUpdateOrd']
	,'program_season': ['TMSId', 'updateId', 'seasonTMSId', 'MD5', 'ingestId']
	,'program_title': ['TMSId', 'updateId', 'title', 'size', 'type', 'subType', 'lang', 'MD5', 'ingestId']
	,'program_description': ['TMSId', 'updateId', 'description', 'size', 'type', 'lang', 'MD5', 'ingestId']
	,'program_cast': ['TMSId', 'updateId', 'ord', 'personId', 'nameId', 'characterName', 'roleId', 'role', 'departed', 'MD5', 'ingestId']
	,'program_crew': ['TMSId', 'updateId', 'ord', 'personId', 'nameId', 'roleId', 'role', 'MD5', 'ingestId']
	,'program_country': ['TMSId', 'updateId', 'country', 'MD5', 'ingestId']
	,'program_award': ['TMSId', 'updateId', 'won', 'award', 'awardId', 'category', 'awardCatId', 'year', 'recipient', 'recipientNameId', 'MD5', 'ingestId']
	,'program_genre': ['TMSId', 'updateId', 'genreOrd', 'genre', 'genreId', 'MD5', 'ingestId']
	,'program_rating': ['TMSId', 'updateId', 'exempt', 'rating_code', 'ratingBodyId', 'ratingsBody', 'warning', 'MD5', 'ingestId']
	,'program_qualityrating': ['TMSId', 'updateId', 'ratingsBody', 'value', 'numVotes', 'MD5', 'ingestId']
	,'program_ratingadvisory': ['TMSId', 'updateId', 'advisory', 'ratingsBody', 'MD5', 'ingestId']
	,'program_mvinfo': ['TMSId', 'updateId', 'key', 'value', 'MD5', 'ingestId']
	,'program_mvtrailer': ['TMSId', 'updateId', 'format', 'URL', 'MD5', 'ingestId']
	,'program_epinfo': ['episodeInfoId', 'TMSId', 'updateId', 'title', 'size', 'type', 'subType', 'lang', 'season', 'number', 'episodes', 'numInSeries', 'seasonYear', 'synNum', 'altSynNum', 'MD5', 'ingestId']
	,'program_epinfo_alt': ['episodeInfoId', 'TMSId', 'updateId', 'country', 'countrywide', 'prgSvcId', 'season', 'episode', 'MD5', 'ingestId']
	,'program_spinfo': ['TMSId', 'updateId', 'gameDate', 'gameTime', 'gameTimeZone', 'venue', 'venueBrandId', 'season', 'season_type', 'playoffRound', 'playoffRound_gameNum', 'MD5', 'ingestId']
	,'program_spteam': ['TMSId', 'updateId', 'team', 'teamBrandId', 'isHome', 'MD5', 'ingestId']
	,'program_spevent': ['TMSId', 'updateId', 'eventId', 'type', 'MD5', 'ingestId']
	,'program_spdata': ['sportsDataId', 'TMSId', 'updateId', 'homeParticipantGId', 'awayParticipantGId', 'participantType', 'sportGId', 'venueGId', 'gender', 'MD5', 'ingestId']
	,'program_spdata_event': ['sportsDataId', 'TMSId', 'updateId', 'sportsEventGId', 'type', 'MD5', 'ingestId']
	,'program_asset': ['programAssetId', 'TMSId', 'updateId', 'assetUpdateId', 'assetId', 'deleted', 'lastModified', 'expiredDate', 'type', 'width', 'height', 'primary', 'category', 'ratio', 'tier', 'seasonId', 'restrictedProvider', 'provider', 'URI', 'caption', 'caption_lang', 'MD5', 'ingestId']
	,'program_asset_id': ['programAssetId', 'TMSId', 'updateId', 'id', 'type', 'MD5', 'ingestId']
	,'program_release': ['releaseId', 'TMSId', 'updateId', 'editNotes', 'country', 'type', 'date', 'medium', 'prgSvcId', 'seasonYear', 'seasonName', 'MD5', 'ingestId']
	,'program_release_attribute': ['releaseId', 'TMSId', 'updateId', 'attribute', 'attribute_type', 'MD5', 'ingestId']
	,'program_release_distributor': ['releaseId', 'TMSId', 'updateId', 'name', 'MD5', 'ingestId']
	,'program_relationship': ['TMSId', 'updateId', 'type', 'association', 'rootId', 'organizationId', 'personId', 'personNameId', 'label', 'MD5', 'ingestId']
	,'program_externallink': ['TMSId', 'updateId', 'vendor', 'vendorId', 'id', 'url', 'MD5', 'ingestId']
	,'program_descriptor': ['TMSId', 'updateId', 'ord', 'descriptorId', 'weight', 'type', 'MD5', 'ingestId']
	,'program_origaudiolang': ['TMSId', 'updateId', 'ord', 'lang', 'MD5', 'ingestId']
	,'season': ['seasonTMSId', 'seasonId', 'updateId', 'updateDate', 'deleted', 'inactive', 'seasonNumber', 'seasonYear', 'seasonPremiere', 'seasonFinale', 'origNetwork', 'origNetwork_prgSvcId', 'totalSeasonEpisodes', 'showcard', 'officialURL', 'title_lang', 'desc_lang', 'MD5', 'ingestId', 'batchUpdateOrd']
	,'season_title': ['seasonTMSId', 'updateId', 'title', 'size', 'type', 'subType', 'lang', 'MD5', 'ingestId']
	,'season_description': ['seasonTMSId', 'updateId', 'description', 'size', 'type', 'lang', 'MD5', 'ingestId']
	,'season_cast': ['seasonTMSId', 'updateId', 'ord', 'personId', 'nameId', 'characterName', 'roleId', 'role', 'departed', 'MD5', 'ingestId']
	,'season_crew': ['seasonTMSId', 'updateId', 'ord', 'personId', 'nameId', 'roleId', 'role', 'MD5', 'ingestId']
	,'season_asset': ['seasonAssetId', 'seasonTMSId', 'updateId', 'assetUpdateId', 'assetId', 'deleted', 'lastModified', 'expiredDate', 'type', 'width', 'height', 'primary', 'category', 'ratio', 'tier', 'restrictedProvider', 'provider', 'seasonId', 'URI', 'caption', 'caption_lang', 'MD5', 'ingestId']
	,'season_asset_id': ['seasonAssetId', 'seasonTMSId', 'updateId', 'id', 'type', 'MD5', 'ingestId']

}


# =================== ENDPOINT SPECIFIC INIT CODE END ===================


gnProgramsParser = GnProgramsParser( endpoint, entities, tables, fields )
gnProgramsParser.set_config( os.environ['GXD_PDK_DIR'], os.environ['GXD_PDK_CONFIG'], files_path )
gnProgramsParser.run( xml_file )
