#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Parses a given ON Universities XML file
#  Validates each record against fields configuration file
#  Writes validated records into TSV files for the endpoint
#  =============================================================================

import os
import sys

sys.path.append(os.path.join(sys.path[0],'..'))
import gxd_functions as gxd

xml_file 	= sys.argv[1]
files_path 	= sys.argv[2]

class GnUniversitiesParser(gxd.GnEndpointParser):

	def parse(self, xml_file):

		for u in self.xml_nodes(xml_file, "on/universities/university"):

			university = {}

			universityId 				= self.check( u, 'universityId')
			updateId 					= self.check( u, 'updateId' )
			updateDate 					= self.check( u, 'updateDate' )

			university['universityId'] 	= universityId
			university['updateId'] 		= updateId
			university['updateDate']	= updateDate

			university['universityGId'] = self.check( u, 'universityGId')
			university['deleted'] 		= self.format_boolean( self.check( u, 'deleted' ) )
			university['name'] 			= self.check( u.find('name') )

			university['names_lang'] 		= self.check( u.find('names'), 'lang' )
			university['address_city'] 		= self.check( u.find('address/city') )
			university['address_state'] 	= self.check( u.find('address/state') )
			university['address_country'] 	= self.check( u.find('address/country') )

			if self.do_output_tsv:
				if not self.validate_and_write_tsv(university, 'university', 'universityId'):
					u.clear()
					continue

			for un in u.findall('names/name'):

				name = {}

				name['universityId']	= universityId
				name['updateId'] 		= updateId
				name['name'] 			= self.check( un )
				name['type'] 			= self.check( un, 'type' )
				name['lang'] 			= self.check( un, 'lang' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(name, 'university_name', 'universityId'):
						un.clear()
						continue

				if self.do_output_jsonl:
					[ name.pop(key) for key in ['universityId','updateId'] ]
					university['names'] = ( university.get('names') or [] ) + [ name.copy() ]

				un.clear()

			for ua in u.findall('assets/asset'):

				asset = {}

				asset['universityId'] 	= universityId
				asset['updateId'] 		= updateId
				asset['assetId'] 		= self.check( ua, 'assetId' )
				asset['assetUpdateId'] 	= self.check( ua, 'updateId' )
				asset['lastModified'] 	= self.format_datetime( self.check( ua, 'lastModified' ))
				asset['expiredDate'] 	= self.check( ua, 'expiredDate' )
				asset['type'] 			= self.check( ua, 'type' )
				asset['width'] 			= self.check( ua, 'width' )
				asset['height'] 		= self.check( ua, 'height' )
				asset['ratio'] 			= self.check( ua, 'ratio' )
				asset['tier']	 		= self.check( ua, 'tier' )
				asset['category'] 		= self.check( ua, 'category' )
				asset['deleted'] 		= self.format_boolean( self.check( ua, 'deleted' ))
				asset['primary'] 		= self.format_boolean( self.check( ua, 'primary' ))
				asset['URI'] 			= self.check( ua.find('URI') )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(asset, 'university_asset', 'universityId'):
						ua.clear()
						continue

				if self.do_output_jsonl:
					[ asset.pop(key) for key in ['universityId','updateId'] ]
					university['assets'] = ( university.get('assets') or [] ) + [ asset.copy() ]

				ua.clear()

			if self.do_output_jsonl:
				self.write_jsonl(university, 'university')

			u.clear()


# ================== ENDPOINT SPECIFIC INIT CODE BEGIN ==================
# Here insert the endpoint-specific code from schema/release/<product>/parse_.py

endpoint = 'Universities'


entities = [
	 'university' 
]

tables = [
	 'university' 
	,'university_name'
	,'university_asset' 
]

fields = {
	 'university': ['universityId', 'updateId', 'updateDate', 'deleted', 'universityGId', 'name', 'names_lang', 'address_city', 'address_state', 'address_country', 'MD5', 'ingestId', 'batchUpdateOrd']
	,'university_name': ['universityId', 'updateId', 'name', 'type', 'lang', 'MD5', 'ingestId']
	,'university_asset': ['universityId', 'updateId', 'assetUpdateId', 'assetId', 'deleted', 'lastModified', 'expiredDate', 'type', 'width', 'height', 'primary', 'category', 'ratio', 'tier', 'URI', 'MD5', 'ingestId']
}


# =================== ENDPOINT SPECIFIC INIT CODE END ===================


gnUniversitiesParser = GnUniversitiesParser( endpoint, entities, tables, fields )
gnUniversitiesParser.set_config( os.environ['GXD_PDK_DIR'], os.environ['GXD_PDK_CONFIG'], files_path )
gnUniversitiesParser.run( xml_file )
