#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Parses a given ON Venues XML file
#  Validates each record against fields configuration file
#  Writes validated records into TSV files for the endpoint
#  =============================================================================

import os
import sys

sys.path.append(os.path.join(sys.path[0],'..'))
import gxd_functions as gxd

xml_file 	= sys.argv[1]
files_path 	= sys.argv[2]

class GnVenuesParser(gxd.GnEndpointParser):

	def parse(self, xml_file):

		for v in self.xml_nodes(xml_file, "on/venues/venue"):

			venue = {}

			venueId 					= self.check( v, 'venueId')
			updateId 					= self.check( v, 'updateId' )
			updateDate 					= self.check( v, 'updateDate' )
			is_deleted					= self.format_boolean( self.check( v, 'deleted' ) )

			venue['venueId'] 			= venueId
			venue['updateId'] 			= updateId
			venue['updateDate']			= updateDate
			venue['deleted'] 			= is_deleted

			venue['venueGId'] 			= self.check( v, 'venueGId' )
			venue['type'] 				= self.check( v.find('type') )
			venue['timezone'] 			= self.check( v.find('timezone') )
			venue['address_city'] 		= self.check( v.find('address/city') )
			venue['address_state'] 		= self.check( v.find('address/state') )
			venue['address_country'] 	= self.check( v.find('address/country') )

			if self.do_output_tsv:
				if not self.validate_and_write_tsv(venue, 'venue', 'venueId'):
					v.clear()
					continue

			for vb in v.findall('venueBrands/venueBrand'):

				venuebrand = {}

				venueBrandId 				= self.check( vb.find('venueBrandId') )

				venuebrand['venueBrandId'] 	= venueBrandId
				venuebrand['venueId'] 		= venueId
				venuebrand['updateId'] 		= updateId
				venuebrand['updateDate']	= updateDate
				venuebrand['deleted']		= is_deleted

				venuebrand['fromDate'] 		= self.format_datetime( self.check( vb.find('fromDate') ), '%Y-%m-%d')
				venuebrand['toDate'] 		= self.format_datetime( self.check( vb.find('toDate')   ), '%Y-%m-%d')
				venuebrand['names_lang'] 	= self.check( vb.find('names'), 'lang' )

				venuebrand['active']	 	= self.format_boolean( self.check( vb.find('active') ) )
				venuebrand['name'] 			= self.check( vb.find('name') )
				venuebrand['name_lang'] 	= self.check( vb.find('name'), 'lang' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(venuebrand, 'venuebrand', 'venueBrandId'):
						vb.clear()
						continue

				for vbn in vb.findall('names/name'):

					name = {}

					name['venueBrandId']	= venueBrandId
					name['updateId'] 		= updateId
					name['name'] 			= self.check( vbn )
					name['type'] 			= self.check( vbn, 'type' )
					name['lang'] 			= self.check( vbn, 'lang' )

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(name, 'venuebrand_name', 'venueBrandId'):
							vbn.clear()
							continue

					if self.do_output_jsonl:
						[ name.pop(key) for key in ['venueBrandId','updateId'] ]
						venuebrand['names'] = ( venuebrand.get('names') or [] ) + [ name.copy() ]

					vbn.clear()

				if self.do_output_jsonl:
					self.write_jsonl(venuebrand, 'venuebrand')

				vb.clear()

			if self.do_output_jsonl:
				self.write_jsonl(venue, 'venue')

			v.clear()


# ================== ENDPOINT SPECIFIC INIT CODE BEGIN ==================
# Here insert the endpoint-specific code from schema/release/<product>/parse_.py

endpoint = 'Venues'


entities = [
	 'venue' 
	,'venuebrand' 
]

tables = [
	 'venue' 
	,'venuebrand' 
	,'venuebrand_name' 
]

fields = {
	 'venue': ['venueId', 'updateId', 'updateDate', 'deleted', 'venueGId', 'type', 'timezone', 'address_city', 'address_state', 'address_country', 'MD5', 'ingestId', 'batchUpdateOrd']
	,'venuebrand': ['venueBrandId', 'updateId', 'updateDate', 'deleted', 'venueId', 'active', 'fromDate', 'toDate', 'names_lang', 'name', 'name_lang', 'MD5', 'ingestId', 'batchUpdateOrd']
	,'venuebrand_name': ['venueBrandId', 'updateId', 'name', 'type', 'lang', 'MD5', 'ingestId']

}


# =================== ENDPOINT SPECIFIC INIT CODE END ===================


gnVenuesParser = GnVenuesParser( endpoint, entities, tables, fields )
gnVenuesParser.set_config( os.environ['GXD_PDK_DIR'], os.environ['GXD_PDK_CONFIG'], files_path )
gnVenuesParser.run( xml_file )
