-- VideoDescriptor retrieval for a TMSId (both vector and matrix/pivot representation)

-- Start with edge layer of descriptors (those assigned, may or may not be taxonomy leaf nodes) 
-- Collect all parent descriptors traversing the tree towards the roots, for max of 6 levels
WITH edge0 AS (
    SELECT pd."TMSId", d."descriptorId", d."parent_descriptorId", pd.weight, 0 AS from_edge
    FROM "descriptor" d JOIN program_descriptor pd ON pd."descriptorId"=d."descriptorId"
    WHERE pd."TMSId" IN ('SH039905020000')
), edge1 AS (
    SELECT e0."TMSId", d."descriptorId", d."parent_descriptorId", e0.weight, 1 AS from_edge
    FROM "descriptor" d JOIN edge0 e0 ON d."descriptorId"=e0."parent_descriptorId"
), edge2 AS (
    SELECT e1."TMSId", d."descriptorId", d."parent_descriptorId", e1.weight, 2 AS from_edge
    FROM "descriptor" d JOIN edge1 e1 ON d."descriptorId"=e1."parent_descriptorId"
), edge3 AS (
    SELECT e2."TMSId", d."descriptorId", d."parent_descriptorId", e2.weight, 3 AS from_edge
    FROM "descriptor" d JOIN edge2 e2 ON d."descriptorId"=e2."parent_descriptorId"
), edge4 AS (
    SELECT e3."TMSId", d."descriptorId", d."parent_descriptorId", e3.weight, 4 AS from_edge
    FROM "descriptor" d JOIN edge3 e3 ON d."descriptorId"=e3."parent_descriptorId"
), edge5 AS (
    SELECT e4."TMSId", d."descriptorId", d."parent_descriptorId", e4.weight, 5 AS from_edge
    FROM "descriptor" d JOIN edge4 e4 ON d."descriptorId"=e4."parent_descriptorId"
)
-- merge accumulated descriptors from all levels
, unaggregated AS (
    SELECT * FROM edge0 UNION
    SELECT * FROM edge1 UNION
    SELECT * FROM edge2 UNION
    SELECT * FROM edge3 UNION
    SELECT * FROM edge4 UNION
    SELECT * FROM edge5
)
-- aggregate the same/duplicate parent descriptors having different (inherited) weights and/or from_edge value:
-- -- pick minimum from_edge value (presumably most relevant) and average weight
-- from here, weight can be additionally modified based on from_edge value, descriptor type etc
-- the output is a flat list / vector representation (ML friendly), containing all applicable descriptors in the hierarchy
, descriptor_vector AS (
	SELECT ua."TMSId"
		,d."typeId", d."typeName"
		,d."descriptorId", d."descriptorName", d."parent_descriptorId"
	    ,ROUND(AVG(weight),1) AS "avg(weight)"
	    ,MIN(from_edge) AS "min(from_edge)"
	FROM unaggregated ua
	    JOIN descriptor d ON ua."descriptorId"=d."descriptorId"
	GROUP BY 1,2,3,4,5,6
	ORDER BY 1,3,6,5 DESC
)
-- SELECT * FROM descriptor_vector;

-- traverse resulting flat list from the root to the edge nodes building the matrix/pivot representation (human friendly)
, pivot_view AS (
	SELECT d1."TMSId"
		,d1."typeId", d1."typeName"
	    ,d1."descriptorName" as "L1_descriptorName"
	    ,d2."descriptorName" as "L2_descriptorName"
	    ,d3."descriptorName" as "L3_descriptorName"
	    ,d4."descriptorName" as "L4_descriptorName"
	    ,d5."descriptorName" as "L5_descriptorName"
	    ,d6."descriptorName" as "L6_descriptorName"
	    ,COALESCE(d6."descriptorId",d5."descriptorId",d4."descriptorId",d3."descriptorId",d2."descriptorId", d1."descriptorId") AS "edge_descriptorId"    
	    ,COUNT(d1."descriptorId")+COUNT(d2."descriptorId")+COUNT(d3."descriptorId")+COUNT(d4."descriptorId")+COUNT(d5."descriptorId")+COUNT(d6."descriptorId")
	    	AS "depth"
	FROM descriptor_vector d1
	    LEFT JOIN descriptor_vector d2 ON d2."parent_descriptorId"=d1."descriptorId"
	    LEFT JOIN descriptor_vector d3 ON d3."parent_descriptorId"=d2."descriptorId"
	    LEFT JOIN descriptor_vector d4 ON d4."parent_descriptorId"=d3."descriptorId"
	    LEFT JOIN descriptor_vector d5 ON d5."parent_descriptorId"=d4."descriptorId"
	    LEFT JOIN descriptor_vector d6 ON d6."parent_descriptorId"=d5."descriptorId"
	WHERE 1=1
	    AND d1."parent_descriptorId" IS NULL
	GROUP BY 1,2,3,4,5,6,7,8,9,10
)
SELECT pv.*, dv."avg(weight)"
FROM pivot_view pv
	JOIN descriptor_vector dv ON pv."edge_descriptorId"=dv."descriptorId" AND pv."TMSId"=dv."TMSId"
ORDER BY "typeName", "avg(weight)" DESC
;
