#!/bin/sh

#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Main script of the Product Demo Kit
#  =============================================================================


BASE_PDK_VER="1.3.22"

. ./bin/functions.sh

trap ctrl_C_handler INT

# Set/create paths/variables
DATETIME=$(date +%Y%m%d%H%M%S)
GXD_INGEST_JOB_ID=$DATETIME
export GXD_PDK_DIR=$(pwd)
export GXD_LOG=$GXD_PDK_DIR/logs/$DATETIME/run.log
export GXD_LOG_DIR=$GXD_PDK_DIR/logs/$DATETIME
export GXD_INGEST_JOB_ID

VARS=$GXD_PDK_DIR/vars

mkdir $GXD_LOG_DIR
mkdir $GXD_LOG_DIR/errors

# Parse command line arguments
while getopts 'a:c:d:fi:s' OPTION; do
	case $OPTION in
		a) CORE_ACTION_OVERRIDE=$OPTARG ;;		# Core action override for delivery mode = NONE
		c) CONFIG=$OPTARG ;;							# Config file overriding default config
		d) DELIVERY_OVERRIDE=$OPTARG ;;			# Delivery mode overriding configured delivery mode
		f) FORCE="TRUE" ;;							# Force resume job if it was abnormally terminated (JOB_STATUS == RUNNING)
		i) FILE_INGEST_TYPE=$OPTARG ;;			# Ingest type (full/incremental/auto) - file products only
		s) SKIP_SCHEMA_VER_CHECK="TRUE" ;;		# Skip relational DB schema version check
	esac
done

# Load config file(s) - default first, then supplied config file (if any)
export GXD_PDK_CONFIG="$CONFIG"
load_config

# Read product name and version
while read LINE; do
	PDK_PRODUCT=$LINE
done < $GXD_PDK_DIR/docs/product.txt
while read LINE; do
	PDK_VER=$LINE
done < $GXD_PDK_DIR/docs/version.txt

echo
msg_ok "$PDK_PRODUCT v$PDK_VER"
log_ok "BASE PDK: v$BASE_PDK_VER"
echo

# If no custom config was specified, output usage instructions
# The job will stop downstream due to missing credentials - unless the default config file was modified
if [ -z "$GXD_PDK_CONFIG" ]; then
	echo "USAGE: ./run.sh -c my.conf [-d SEED|SEED_OR_API]"
	echo
	msg_ok "No config file given, using default config file"
	echo
fi

# Check if any required variables have not been supplied
if [ -z $PYTHON ] ||
   [ -z $DELIVERY ] ||
   [ -z $DATABASE ]
   [ -z $SCHEMA_PREFIX ]
then
	msg_err "ERROR: One or more variables: PYTHON | DELIVERY | DATABASE | SCHEMA_PREFIX is not set in config"
	exit 1
fi

# Check if database credentials/params have not been supplied
if [ $DATABASE = "SQLITE" ]; then
	if [ -z $SQLITE_DIR ]; then
		msg_err "ERROR: One or more variables: SQLITE_DIR is not set in config"
		exit 1
	fi
elif [ $DATABASE = "PGSQL" ]; then
	if [ -z $PGSQL_HOST ] ||
	   [ -z $PGSQL_PORT ] ||
	   [ -z $PGSQL_USER ] ||
	   [ -z $PGSQL_PASS ] ||
	   [ -z $PGSQL_DBNAME ] 
	then
		msg_err "ERROR: One or more variables: PGSQL_HOST | PGSQL_PORT | PGSQL_USER | PGSQL_PASS | PGSQL_DBNAME is not set in config"
		exit 1
	fi
elif [ $DATABASE = "MYSQL" ]; then
	if [ -z $MYSQL_HOST ] ||
	   [ -z $MYSQL_PORT ] ||
	   [ -z $MYSQL_USER ] ||
	   [ -z $MYSQL_PASS ]
	then
		msg_err "ERROR: One or more variables: MYSQL_HOST | MYSQL_PORT | MYSQL_USER | MYSQL_PASS is not set in config"
		exit 1
	fi
elif [ $DATABASE = "MONGO" ]; then
	if [ -z $MONGO_HOST ] || 
	   [ -z $MONGO_PORT ] ||
	   [ -z $MONGO_USER ] || 
	   [ -z $MONGO_PASS ]
	then
		msg_err "ERROR: One or more variables: MONGO_HOST | MONGO_PORT | MONGO_USER | MONGO_PASS is not set in config"
		exit 1
	fi
fi

# Check for system dependencies
for DEPENDENCY in "curl" "xmllint" "$PYTHON"; do
	check_for $DEPENDENCY
done
if [ $DATABASE = "SQLITE" ]; then
	check_for "sqlite3"
fi

# Check for Python dependencies
$PYTHON bin/check_modules.py $PYTHON $DATABASE
if [ $? != 0 ]; then
	echo
	exit 1
fi

# Relational DB: check schema backward compatibility
if  [ $DATABASE = "PGSQL" ] ||
	[ $DATABASE = "MYSQL" ] ||
	[ $DATABASE = "SQLITE" ]
then
	if [ -z "$SKIP_SCHEMA_VER_CHECK" ]; then
		$GXD_PDK_DIR/bin/check_rdb_schema.sh
		if [ $? != 0 ]; then
			echo
			exit 1
		fi
	else
		msg_ok "Skipping DB schema version check"
		echo
	fi
fi

# File products only: check if we have request for incremental ingest on cold start
if  [ "$FILE_INGEST_TYPE" = "INCR" ] &&
	[ ! -e $VARS/last_good_ingest_id.var ]
then
	msg_err "ERROR: Incremental ingest is requested but no previous jobs have successfully completed"
	exit 1	
fi


# Get last job status
JOB_STATUS="OK"
if [ -e $VARS/job_status.var ]; then
	. $VARS/job_status.var
else
	# Create default variable file so it could still be overwritten in the disk full scenario - will this work on any FS?
	echo "JOB_STATUS=$JOB_STATUS" > $VARS/job_status.var
fi

# Get last job id, if any
if [ -e $VARS/job_id.var ]; then
	. $VARS/job_id.var
else
	JOB_ID="NONE"
fi

# Get last job phase, if any
JOB_PHASE="NONE"
if [ -e $VARS/job_phase.var ]; then
	. $VARS/job_phase.var
fi

# Check if last job is still running (or if it was killed)
if [ $JOB_STATUS = "RUNNING" ]; then
	if [ "$FORCE" = "TRUE" ]; then
		log_ok "Using force option"
		JOB_STATUS="ERROR"
	else
		msg_ok "Exiting because ingest job $JOB_ID is still running or was abnormally terminated. To see the job status: $ tail logs/$JOB_ID/run.log"
		echo   "If you're certain that the job is no longer running (note the timestamps at log tail), you can try resuming via force option -f"
		echo   "WARNING: Forcing another job while previous job is still running will compromise data integrity and require re-seeding the dataset"
		echo
		echo
		exit 0
	fi
fi

# Set current job status
echo "JOB_STATUS=RUNNING" > $VARS/job_status.var
echo "JOB_ID=$GXD_INGEST_JOB_ID" > $VARS/job_id.var
msg_ok "Starting ingest job $GXD_INGEST_JOB_ID with configuration file '$GXD_PDK_CONFIG'"

if [ $JOB_STATUS = "ERROR" ]; then
	msg_ok "Previous job $JOB_ID failed, skipping to failed phase"
fi

echo

# As a precaution, check if we're starting from update_id zero but have temporary XML/TSV files (which shoudln't happen) and in this case, output a warning
UID_VAR_COUNT=$( find $GXD_PDK_DIR/vars -maxdepth 1 -type f -name 'update_id_*.var' | wc -l )
XML_COUNT=$( find $GXD_PDK_DIR/files/xml -maxdepth 2 -type f -name '*.xml' | wc -l )
TSV_COUNT=$( find $GXD_PDK_DIR/files/tsv -maxdepth 2 -type f -name '*.tsv' | wc -l )
if [ $UID_VAR_COUNT = 0 ]; then
	if [ $XML_COUNT != 0 ] || [ $TSV_COUNT != 0 ]; then
		echo "WARNING: Found existing XML and/or TSV files while vars/update_id_*.var files do not exist"
		sleep 1
	fi
fi


# Main sequence

ERROR=0

# Download
if [ $ERROR = 0 ] && [ "$PHASE_DOWNLOAD" != "FALSE" ]; then

	# Proceed with download if last job succeeded or failed in download phase
	if [ $JOB_STATUS = "OK" ] || [ $JOB_PHASE = "DOWNLOAD" ]; then
		time -p $GXD_PDK_DIR/bin/download.sh $JOB_STATUS "$DELIVERY_OVERRIDE" "$CORE_ACTION_OVERRIDE" "$FILE_INGEST_TYPE"
		ERROR=$?
		# Check whether previously failing download phase has now finished successfully and if so, reset the last job status
		if [ $ERROR = 0 ]; then
			if 	[ $JOB_PHASE = "DOWNLOAD" ]; then
				JOB_STATUS="OK"
			fi
		else
			# Will exit_abnormal at the end of the script since ERROR is not 0
			msg_err "ERROR: download phase failed, rectify error and re-run"
		fi
	fi
fi


# Parse
if [ $ERROR = 0 ] && [ "$PHASE_PARSE" != "FALSE" ]; then

	# Proceed with parsing if last job succeeded or failed in parsing phase
	if [ $JOB_STATUS = "OK" ] || [ $JOB_PHASE = "PARSE" ]; then
		time -p $GXD_PDK_DIR/bin/parse.sh #$JOB_STATUS
		ERROR=$?
		if [ $ERROR = 0 ]; then
			if [ $JOB_PHASE = "PARSE" ]; then
				JOB_STATUS="OK"
			fi
		else
			msg_err "ERROR: parse phase failed, rectify error and re-run"
		fi
	fi
fi

# Inserts
if [ $ERROR = 0 ] && [ "$PHASE_INSERT" != "FALSE" ]; then

	# Proceed with inserts if last job succeeded (JOB_STATUS=OK) or failed in inserts phase (JOB_STATUS=ERROR)
	if [ $JOB_STATUS = "OK" ] || [ $JOB_PHASE = "INSERT" ]; then
		if [ $DATABASE = "SQLITE" ]; then
			time -p $GXD_PDK_DIR/bin/db_sqlite/inserts.sh $JOB_STATUS
		elif [ $DATABASE = "PGSQL" ]; then
			time -p $GXD_PDK_DIR/bin/db_pgsql/inserts.sh $JOB_STATUS
		elif [ $DATABASE = "MYSQL" ]; then
			time -p $GXD_PDK_DIR/bin/db_mysql/inserts.sh $JOB_STATUS
		elif [ $DATABASE = "MONGO" ]; then
			time -p $GXD_PDK_DIR/bin/db_mongo/inserts.sh $JOB_STATUS
		fi
		ERROR=$?
		if [ $ERROR = 0 ]; then
			if [ $JOB_PHASE = "INSERT" ]; then
				JOB_STATUS="OK"
			fi
		else
			msg_err "INSERT: ERROR: insert phase failed, rectify error and re-run"
		fi
	fi
fi

# Stage
if [ $ERROR = 0 ] && [ "$PHASE_STAGE" != "FALSE" ]; then

	# Proceed with staging if last job succeeded (JOB_STATUS=OK) or failed in staging phase (JOB_STATUS=ERROR)
	if [ $JOB_STATUS = "OK" ] || [ $JOB_PHASE = "STAGE" ]; then
		if [ $DATABASE = "SQLITE" ]; then
			time -p $GXD_PDK_DIR/bin/db_sqlite/stage.sh $JOB_STATUS
		elif [ $DATABASE = "PGSQL" ]; then
			time -p $GXD_PDK_DIR/bin/db_pgsql/stage.sh $JOB_STATUS
		elif [ $DATABASE = "MYSQL" ]; then
			time -p $GXD_PDK_DIR/bin/db_mysql/stage.sh $JOB_STATUS
		elif [ $DATABASE = "MONGO" ]; then
			time -p $GXD_PDK_DIR/bin/db_mongo/stage.sh $JOB_STATUS
		fi
		ERROR=$?
		if [ $ERROR = 0 ]; then
			if [ $JOB_PHASE = "STAGE" ]; then
				JOB_STATUS="OK"
			fi
		else
			msg_err "STAGE: ERROR: stage phase failed, rectify error and re-run"
		fi
	fi
fi

# Core
if [ $ERROR = 0 ] && [ "$PHASE_CORE" != "FALSE" ]; then

	# Retrieve upstream (or previous job's) decision to update or replace core
	if [ -e $VARS/core_action.var ]; then
		. $VARS/core_action.var	
	else
		CORE_ACTION="REPLACE"
	fi

	# Either proceed with core replace if last job succeeded (JOB_STATUS=OK) or failed in core replace phase (JOB_STATUS=ERROR)
	if [ $CORE_ACTION = "REPLACE" ]; then

		if [ $JOB_STATUS = "OK" ] || [ $JOB_PHASE = "CORE_REPLACE" ]; then

			if [ $DATABASE = "SQLITE" ]; then
				time -p $GXD_PDK_DIR/bin/db_sqlite/replace_core.sh $JOB_STATUS
			elif [ $DATABASE = "PGSQL" ]; then
				time -p $GXD_PDK_DIR/bin/db_pgsql/replace_core.sh $JOB_STATUS
			elif [ $DATABASE = "MYSQL" ]; then
				time -p $GXD_PDK_DIR/bin/db_mysql/replace_core.sh $JOB_STATUS
			elif [ $DATABASE = "MONGO" ]; then
				time -p $GXD_PDK_DIR/bin/db_mongo/replace_core.sh $JOB_STATUS
			fi
		fi

	# Or proceed with core update if last job succeeded (JOB_STATUS=OK) or failed in core update phase (JOB_STATUS=ERROR)
	elif [ $CORE_ACTION = "UPDATE" ]; then

		if [ $JOB_STATUS = "OK" ] || [ $JOB_PHASE = "CORE_UPDATE" ]; then

			if [ $DATABASE = "SQLITE" ]; then
				time -p $GXD_PDK_DIR/bin/db_sqlite/update_core.sh $JOB_STATUS
			elif [ $DATABASE = "PGSQL" ]; then
				time -p $GXD_PDK_DIR/bin/db_pgsql/update_core.sh $JOB_STATUS
			elif [ $DATABASE = "MYSQL" ]; then
				time -p $GXD_PDK_DIR/bin/db_mysql/update_core.sh $JOB_STATUS
			elif [ $DATABASE = "MONGO" ]; then
				time -p $GXD_PDK_DIR/bin/db_mongo/update_core.sh $JOB_STATUS
			fi

		fi

	fi

	ERROR=$?

	if [ $ERROR = 0 ]; then
		if [ $JOB_PHASE = "CORE_REPLACE" ] ||
		   [ $JOB_PHASE = "CORE_UPDATE" ] 
		then
			JOB_STATUS="OK"
		fi
	else
		msg_err "CORE: ERROR: core update or replace phase failed, rectify error and re-run"
	fi	

fi

# Complete
if [ $ERROR = 0 ]; then
	rm -f $VARS/job_phase.var
	echo "LAST_GOOD_INGEST_ID=$GXD_INGEST_JOB_ID" > $VARS/last_good_ingest_id.var

	echo
	msg_ok "Ingest job $GXD_INGEST_JOB_ID complete"
	echo

	exit_normal
else
	exit_abnormal
fi
