import os
import sys
import csv
from pathlib import Path
from collections import OrderedDict

out_file 	= None
sql_flavor	= None
#tables 		= None

# TODO: make this a fallback while getting the value from passed parameter (originally from config)
sqlite_db_path_stub = '__gxd_runsql_sqlite_db_path_stub__'


def str2bool(v):
  return v.lower() in ("yes", "true", "t", "1")		

def empty_value(field_type):
	if field_type == 'VARCHAR':
		return "''"
	if field_type in ['INT','BIGINT','SMALLINT']:
		return 0
	if field_type == 'DATETIME':
		return "'1900-01-01 00:00:00'"
	if field_type == 'FLOAT':
		return 0.0
	if field_type == 'BOOL':
		return "'FALSE'"

# Iterate returning list of field items for each table
def fields_by_table(fields):
	table = None
	tf = []
	for f in fields:
		if f['table'] == table:
			tf.append(f)
		else:
			if table is not None:
				yield tf
			table = f['table']
			tf = [f]
	yield tf

# Build dict with significant PDK field names per table
def build_pdk_function_dict(fields):
	tables = OrderedDict()
	for fields in fields_by_table(fields):
		table = fields[0]['table']

		tables[ table ] = { 'FIELD_NAMES':[], 'FKs':[], 'OTHER_NOT_NULLS':[] }
		for f in fields:
			pdk_function_string = f['pdk_function']
			pdk_function_primary = pdk_function_string.split('|')[0]
			pdk_function_key = pdk_function_primary.split(':')[0]
			pdk_function_val = pdk_function_primary.split(':')[1] if len(pdk_function_primary.split(':')) > 1 else None

			if pdk_function_key == 'PRIMARY_KEY':
				tables[ table ][ 'PRIMARY_KEY' ] = f['name']

			# Non-parent table - fields parent entity reference
			if pdk_function_key == 'PARENT_ENTITY' or pdk_function_key == 'PARENT':
				tables[ table ][ 'PARENT_ENT_REF' ] = f['name']
				tables[ table ][ 'PARENT_ENT' ] = pdk_function_val

			# Grandchild table -  parent table reference - not used since triggers were removed
			if pdk_function_key == 'PARENT_TABLE' or pdk_function_key == 'PARENT':
				tables[ table ][ 'PARENT_TBL_REF' ] = f['name']
				tables[ table ][ 'PARENT_TBL' ] = pdk_function_val

			# Delete flag for dependent entities
			if pdk_function_key == 'DELETE_FLAG':
				tables[ table ][ 'DELETE_FLAG' ] = f['name']
				tables[ table ][ 'DELETE_FLAG_RELATION_TABLE' ] = pdk_function_val

			# Collect foreign keys to process constraints
			if pdk_function_key == 'FOREIGN_KEY':
				tables[ table ][ 'FKs' ].append({ f['name'] : pdk_function_val })

			# Collect non-nullable fields to reconstruct/insert the delete/inactive signal for decomposed entities (parent table only)
			if  not tables[table].get('PARENT_ENT_REF') and not str2bool(f['nullable']) and pdk_function_key not in ['PRIMARY_KEY','UPDATE_ID','UPDATE_DATE']:
				tables[ table ][ 'OTHER_NOT_NULLS' ].append({ f['name'] : f['type'] })

			# Assign Update + Ingest id fields
			if pdk_function_key == 'UPDATE_ID':
				tables[ table ][ 'UPDATE_ID' ] = f['name']
			if pdk_function_key == 'UPDATE_DATE':
				tables[ table ][ 'UPDATE_DATE' ] = f['name']
				tables[ table ][ 'UPDATE_DATE_STATS' ] = pdk_function_val			
			if pdk_function_key == 'RESOURCE':
				tables[ table ][ 'RESOURCE' ] = f['name']
			if pdk_function_key == 'INACTIVE':
				tables[ table ][ 'INACTIVE' ] = f['name']
			if pdk_function_key == 'INGEST_ID':
				tables[ table ][ 'INGEST_ID' ] = f['name']
				tables[ table ][ 'INGEST_ID_STATS' ] = pdk_function_val			
			if pdk_function_key == 'ROW_HASH' and pdk_function_val == 'dedupe':
				tables[ table ][ 'ROW_HASH' ] = f['name']
			if pdk_function_key == 'BATCH_UPDATE_ORD':
				tables[ table ][ 'BATCH_UPDATE_ORD' ] = f['name']

			tables[ table ][ 'FIELD_NAMES' ].append( f['name'] )
#		print(table, tables[ table ])
#	print()

	return tables




######### SQL porting functions #########


# Backticks substitution for MySQL
def write(file, text):
	global sql_flavor
	file.write(text.replace('"', '`' if sql_flavor=="mysql" else '"' ))

# CTE table from values
def values(values):
	global sql_flavor
	if sql_flavor=='mysql':
		values = [ "ROW('"+v+"')"for v in values.split('|') ]
	else:
		values = [ "('"+v+"')"for v in values.split('|') ]
	return 'VALUES '+','.join(values)

# Concat (defunct)
def concat_ws(sep, values):
	return f"CONCAT_WS('{sep}',{','.join(values)})"

# Now timestamp
def now():
	global sql_flavor

	if sql_flavor == 'sqlite':
		return "DATETIME('NOW')"
	elif sql_flavor == 'mysql':
		return "NOW()"
	elif sql_flavor == 'pgsql':
		return "DATE_TRUNC('second', CURRENT_TIMESTAMP)"

# Today's date
def today():
	global sql_flavor

	if sql_flavor == 'sqlite':
		return "DATE('NOW')"
	elif sql_flavor == 'mysql':
		return "DATE(NOW())"
	elif sql_flavor == 'pgsql':
		return "CURRENT_DATE"

# Interval
def interval(value, unit):
	global sql_flavor

	sign = '+' if value >= 0 else '-'

	if sql_flavor == 'sqlite':
		return f"'{value} {unit}'"

	elif sql_flavor == 'mysql':
		return f"{sign}INTERVAL {abs(value)} {unit}"

	elif sql_flavor == 'pgsql':
		return f"{sign}'{abs(value)}{unit}'::INTERVAL"


# Date with interval: SQLite does not support INTERVAL, instead DATE/DATETIME must be called with interval as one of parameters
def date_plus_interval(date='today', ival=0, unit='day'):
	global sql_flavor

	date_str = today() if date=='today' else date

	if sql_flavor == 'sqlite':
		return f"DATE({date_str})" if ival == 0 else f"DATE({date_str}, {interval(ival, unit)})"

	elif sql_flavor == 'mysql':
		return f"DATE({date_str})" if ival == 0 else f"{date_str} {interval(ival, unit)}"

	elif sql_flavor == 'pgsql':
		return f"{date_str}" if ival == 0 else f"{date_str} {interval(ival, unit)}"


# UNIX epoch
def epoch(datetime):
	global sql_flavor

	if sql_flavor == 'sqlite':
		return f"UNIXEPOCH({datetime})"

	elif sql_flavor == 'mysql':
		return f"UNIX_TIMESTAMP({datetime})"

	elif sql_flavor == 'pgsql':
		return f"EXTRACT (EPOCH FROM {datetime})"


# Typecasting
def cast_as(value,type):
	global sql_flavor

	sqlite_types = {'int': 'INTEGER' }
	mysql_types = {'int': 'UNSIGNED', 'date': 'DATE'}
	pgsql_types = {'int': 'INTEGER', 'date': 'DATE'}

	if sql_flavor == 'sqlite':
		if type=='date':
			return f'DATE({value})'
		else:
			return f'CAST({value} AS {sqlite_types.get(type)})'

	elif sql_flavor == 'mysql':
		return f'CAST({value} AS {mysql_types.get(type)})'

	elif sql_flavor == 'pgsql':
		return f'{value}::{pgsql_types.get(type)}'



######### 


def stats_header(out_file):
	global sqlite_db_path_stub
	# Parameters for runsql.py
	schema_core = "{0}"

	pgsql = f"""
-- {schema_core}: core schema

SET SEARCH_PATH TO '{schema_core}';

CREATE TABLE IF NOT EXISTS _stats (
	"timestamp" timestamp without time zone NOT NULL,
	"metric" character varying(255) NOT NULL,
	"dimension" character varying(255),
	"value" real );

CREATE INDEX IF NOT EXISTS "_stats_timestamp" ON {schema_core}._stats USING btree ("timestamp");
CREATE INDEX IF NOT EXISTS "_stats_metric" ON {schema_core}._stats USING btree ("metric");

"""

	mysql = f"""
-- {schema_core}: core schema

USE {schema_core};

CREATE TABLE IF NOT EXISTS `_stats` (
	`timestamp` datetime NOT NULL,
	`metric` varchar(255) NOT NULL,
	`dimension` varchar(255),
	`value` real,
	KEY `_stats_timestamp` (`timestamp`),
	KEY `_stats_metric` (`metric`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

"""

	sqlite = f"""

-- Stats requires SQLite version 3.44.1 (2023-11-22) or above

-- {schema_core}: core schema

ATTACH "{sqlite_db_path_stub}{schema_core}.db" AS core;

CREATE TABLE IF NOT EXISTS core._stats (
	timestamp DATETIME NOT NULL,
	metric VARCHAR(255) NOT NULL,
	dimension VARCHAR(255),
	value REAL );

CREATE INDEX IF NOT EXISTS core._stats_timestamp ON _stats (timestamp);
CREATE INDEX IF NOT EXISTS core._stats_metric ON _stats (metric);

"""

	with open(f"{out_file}", "w") as out:		# opening to write, all subsequent writes will append
		if sql_flavor=='pgsql':
			write(out, pgsql)

		elif sql_flavor=='sqlite':
			write(out, sqlite)

		elif sql_flavor=='mysql':
			write(out, mysql)


def stats_ingests(out_file, entity, key, dimension=None):

	if dimension is None:
		metric = f'{entity}_ingests'
		dimension = 'NULL'
	else:
		metric = f'{entity}{{{dimension}}}_ingests'
		dimension = f'"{dimension}"'


	if sql_flavor=='pgsql':
		timestamp = f"""TO_TIMESTAMP( "ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' )"""

	if sql_flavor=='mysql':
		timestamp = f"""STR_TO_DATE( CAST("ingestId" AS CHAR(14)), '%Y%m%d%H%i%s' )"""

	if sql_flavor=='sqlite':
		timestamp = f"""DATETIME(
		substr("ingestId",1,4)||'-'||substr("ingestId",5,2)||'-'||substr("ingestId",7,2)||' '||
		substr("ingestId",9,2)||':'||substr("ingestId",11,2)||':'||substr("ingestId",13,2) )"""


	with open(f"{out_file}", "a") as out:
		write( out, f"""
-- ingest_id stats for {entity}
INSERT INTO _stats ("timestamp", "metric", "dimension", "value")
SELECT 
	{timestamp} AS "timestamp"
    ,'{metric}' AS metric
    ,{dimension} AS "dimension"
    ,COUNT("{key}") AS value
FROM {entity}
WHERE {timestamp} 
	> (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM _stats WHERE metric = '{metric}')
GROUP BY 1,2,3 ORDER BY 1 DESC
;
		""" )



def stats_updates(out_file, entity, key, dimension=None, interval=900):

	if dimension is None:
		metric = f'{entity}_updates'
		dimension = 'NULL'
	else:
		metric = f'{entity}{{{dimension}}}_updates'
		dimension = f'"{dimension}"'


	if sql_flavor=='pgsql':
		timestamp = f"""
	TO_TIMESTAMP(
		CEILING(
			EXTRACT(EPOCH FROM "updateDate") / {interval} 
		) * {interval})::TIMESTAMP AS "timestamp"	"""

	elif sql_flavor=='mysql':
		timestamp = f"""
	FROM_UNIXTIME(
		CEILING(
			UNIX_TIMESTAMP("updateDate") / {interval} 
		) * {interval}) AS "timestamp"	"""

	elif sql_flavor=='sqlite':
		timestamp = f"""
	-- SQLite: emulating CEILING() with INT(epoch/{interval}) + (epoch/{interval} > INT(epoch/{interval}))
	DATETIME((
		UNIXEPOCH("updateDate")/{interval} + ( UNIXEPOCH("updateDate")/{interval}.0 > UNIXEPOCH("updateDate")/{interval} )
		) * {interval},'unixepoch') AS "timestamp"	"""


	with open(f"{out_file}", "a") as out:
		write( out, f"""
-- update_date stats for {entity}
INSERT INTO _stats ("timestamp", "metric", "dimension", "value")
SELECT {timestamp}
    ,'{metric}' AS metric
    ,{dimension} AS "dimension"    
    ,COUNT("{key}") AS value
FROM {entity}
WHERE "updateDate" > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM _stats WHERE metric = '{metric}')
GROUP BY 1,2,3 ORDER BY 1 DESC
;
		""" )


# This function computes and inserts aggregated table sizes for each entity, as well as for stats and the total size
def stats_db_size(out_file):

	schema_core = "{0}"

	pgsql = """

WITH table_sizes AS (
SELECT DATE_TRUNC('second', CURRENT_TIMESTAMP) AS "timestamp"
		,table_schema
		,SPLIT_PART(table_name,'_',1) AS entity, table_name
		,pg_total_relation_size('"' || table_schema || '"."' || table_name || '"') AS size
	FROM information_schema.tables
)
INSERT INTO _stats ("timestamp", "metric", "dimension", "value")

SELECT "timestamp", 'db_size{{entity}}' AS "metric", entity AS "dimension", SUM(size) AS "value"
FROM table_sizes
WHERE table_schema IN ('{0}') AND "entity" != ''
GROUP BY 1,2,3

UNION SELECT "timestamp", 'db_size{{entity}}' AS "metric", 'stats' AS "dimension", SUM(size) AS "value"
FROM table_sizes
WHERE table_schema IN ('{0}') AND "entity" = ''
GROUP BY 1,2,3

UNION SELECT "timestamp", 'db_size{{entity}}' AS "metric", 'all' AS "dimension", SUM(size) AS "value"
FROM table_sizes
WHERE table_schema IN ('{0}')
GROUP BY 1,2,3

;
	""".format(schema_core)

	mysql = """

INSERT INTO _stats ("timestamp", "metric", "dimension", "value")

WITH table_sizes AS (
	SELECT NOW() AS "timestamp"
		,SUBSTRING_INDEX("table_name", '_', 1) AS "entity"
		,"table_name"
		,data_length + index_length AS "size"
	FROM information_schema.TABLES 
	WHERE table_schema IN ('{0}')
)

SELECT "timestamp", 'db_size{{entity}}' AS "metric", "entity" AS "dimension", SUM("size") AS "value"
FROM table_sizes
WHERE "entity" != ''
GROUP BY 1,2,3

UNION SELECT "timestamp", 'db_size{{entity}}' AS "metric", 'stats' AS "dimension", SUM("size") AS "value"
FROM table_sizes
WHERE "entity" = ''
GROUP BY 1,2,3

UNION SELECT "timestamp", 'db_size{{entity}}' AS "metric", 'all' AS "dimension", SUM("size") AS "value"
FROM table_sizes
GROUP BY 1,2,3
;

""".format(schema_core)

	sqlite = """

WITH ts AS (SELECT DATETIME('NOW') AS "timestamp")
,sz AS (SELECT * FROM pragma_page_size), cnt AS (SELECT * FROM pragma_page_count)
,entity_sizes AS (
	SELECT ts."timestamp"
		,SUBSTR("name", 0, COALESCE(NULLIF(INSTR("name", '_'),0),LENGTH("name")+1)) AS "entity"
		,SUM("pgsize") AS "size"
	FROM dbstat CROSS JOIN ts
	GROUP BY 1,2
	HAVING "entity"!='sqlite'
)
INSERT INTO _stats ("timestamp", "metric", "dimension", "value")

SELECT "timestamp", 'db_size{entity}' AS "metric", "entity" AS "dimension", "size" AS "value"
FROM entity_sizes WHERE "entity" != ''

UNION SELECT "timestamp", 'db_size{entity}' AS "metric", 'stats' AS "dimension", "size" AS "value"
FROM entity_sizes WHERE "entity" = ''

UNION SELECT ts."timestamp", 'db_size{entity}' AS "metric", 'all' AS "dimension", sz.page_size * cnt.page_count AS "value" 
FROM sz CROSS JOIN cnt CROSS JOIN ts
;

	"""

	# write db size calcs to stats file
	with open(f"{out_file}", "a") as out:

		if sql_flavor=='pgsql':
			write(out, pgsql)

		elif sql_flavor=='sqlite':
			write(out, sqlite)

		elif sql_flavor=='mysql':
			write(out, mysql)



# SQL to get timestamp and insert data into _stats table (final step - insert)
def stats_insert_into_sql():
	sql =	f'INSERT INTO _stats ("timestamp", "metric", "dimension", "value")\n'
	return sql

# SQL to get timestamp and insert data into _stats table (final step - select)
def stats_insert_select_sql(from_table):
	sql =	f'SELECT {now()} AS "timestamp"\n' \
			f'	,"metric", "dimension", "value"\n' \
			f'FROM "{from_table}"\n' \
			f';\n'
	return sql		


# Writes out stats query using arbitrary SQL
def stats_insert(out_file, comment, raw_sql, from_table):
	global sql_flavor

	with open(f"{out_file}", "a") as out:		# opening to append, not write!

		write( out,	f'\n-- {comment}\n' )

		# Handle different CTE insert order in MySQL (INSERT.. WITH.. SELECT..) vs standard (WITH.. INSERT.. SELECT..)
		if sql_flavor == 'mysql':
			write( out, stats_insert_into_sql() )

		write( out, raw_sql )

		# Handle different CTE insert order in MySQL (INSERT.. WITH.. SELECT..) vs standard (WITH.. INSERT.. SELECT..)
		if sql_flavor != 'mysql':
			write( out, stats_insert_into_sql() )

		write( out, stats_insert_select_sql( from_table ) )


# = OVERVIEW =
# This function takes a precursor SQL statement selecting "semi-aggregated" raw counts and writes a complete SQL to insert the stats
# 	The "semi-aggregated" counts are aggregated by the columns(s) in the *dimension* field as well as (optionally), 
#	by the deleted/inactive/missing flag values. These values are then used to infer the "status" and include it in the aggegation

# = PRECURSOR SQL =
# The precursor SQL statement (raw_counts_sql) must produce at least "metric", "dimension", "count" fields; status flags are optional; column order arbitrary
# More than one dimension columns can be supplied (use with merge_dims option). Metric name should be full/final metric name.
# The additional STATUS dimension (total|active|deleted|inactive|missing) can be inferred from the status flags and prepended (via pipe) to the *dimension* field output
# The raw_counts_sql flag columns for missing, deleted, inactive should contain either 1 or NULL; total and active dimensions are calculated automatically
# The flag column names should be supplied to f_... parameters of the function
#	Example raw_counts_sql output below:
#	* NULL is '-', flag name parameters are f_deleted='deleted', f_inactive='inactive', status=True, metric='programs{STATUS}'
#
#	metric 				|dimension 	|count 	|deleted|inactive
#	---------------------------------------------------------
#	programs{STATUS}	|-			|1000	|- 		|-
#	programs{STATUS}	|-			|10		|1 		|-
#	programs{STATUS}	|-			|100	|- 		|1

# = ZERO BACKFILL OPTIONS =
# In order to backfill data with zeros (i.e. to populate all dimension combinations, not just those existing in data),
# dimensions' values and the metric name must be provided as function parameters (dims and metric respectively)
# 	Dimensions' values are provided per dimension, either as a fixed string, field name in raw_counts_sql, or a reference to another metric
#		Dims are passed as list of dicts, 'dim' value is case insensitive i.e. 'uniqueName1' is the same as 'UNIQUENAME1'
#		[ {'dim':'uniqueName1', 'values':'pipe|separated|string'},
#		  {'dim':'uniquename2', 'field':'a_field_in_raw_counts_output'},
#		  {'dim':'UNIQUENAME3', 'metric':'some_precomputed_metric{dimension}'} ]
#	Metric name is the final name of the metric i.e. must include leading 'STATUS' if status output is required
#
# Refer to examples in the main() function

def stats_counts(out_file, comment, raw_counts_sql
				,status=False, status_total=True, f_deleted=None, f_inactive=None, f_missing=None
				,dims=None, merge_dims=False, metric=None
				,sql_has_CTEs=False, f_value='count' ):

	do_pad_with_zeros = False

	# To include 0 values in final counts we will build a list of all value combinations for given dimensions and then left join it with actual counts
	# If no dims are given (or they are invalid), the dim combinations not existing in the data (0 values) will not be included in final counts
	if dims is not None:
		if set([type(dim) for dim in dims]) == {type({})}:

			do_pad_with_zeros = True
			
			dimensions, f_names, t_names = [],[],[]
			for dim in dims:
				name = dim.get('dim')
				f_names.append(f"{name}")
				t_names.append(f"dim_{name.lower()}")
				if dim.get('values') is not None:
					dimensions.append( f"dim_{name.lower()}(\"{name}\") AS ({values(dim.get('values'))})" )
				if dim.get('field') is not None:
					dimensions.append( f"dim_{name.lower()} AS (SELECT DISTINCT \"{dim.get('field')}\" AS \"{name}\" FROM raw_counts)" )
				if dim.get('metric') is not None:
					dimensions.append( f"dim_{name.lower()} AS (SELECT DISTINCT \"dimension\" AS \"{name}\" FROM _stats WHERE metric='{dim.get('metric')}')" )

	sql = ''

	if sql_has_CTEs:
		sql +=	f'{raw_counts_sql}\n' \
				f',\n'
	else:
		# SQL for "raw" counts provided as input parameter, aggregated by dimension(s) and optionally del/inact/missing flags
		sql +=	f'WITH raw_counts AS (\n' \
				f'	{raw_counts_sql}\n' \
				f'),\n'

	# SQL to derive the list of all dimension combinations 
	if do_pad_with_zeros:
		# SQL (prepared above) to extract dimensions from a string or a previously computed metric
		sql +=	f',\n'.join(dimensions)
		sql +=	f',\n'

		# SQL computing cross join matrix (dim_1 x dim_2 ... dim_N etc)
		qf_names = [ '"'+n+'"' for n in f_names ]
		field_names = f'CONCAT_WS(\'|\',{ ",".join(qf_names) })'
		table_names = ' CROSS JOIN '.join(t_names)

		sql +=	f'all_dimension_combinations AS ( \n' \
				f'	SELECT {field_names} AS "dimension" \n' \
				f'	FROM {table_names} \n' \
				f'), \n'

	if merge_dims:
		dimension = field_names #'"||\'|\'||"'.join(f_names)
	else:
		dimension = '"dimension"'

	# SQL to optionally aggregate counts by record "status" (from aggreagtion by del/inact/missing), "status" is then prepended to dimension
	sql +=		f'counts AS (\n'
	if status:
		sql +=	f'	SELECT NULL AS "metric", NULL AS "dimension", NULL AS "value" WHERE FALSE\n'

		if status_total:
			sql +=	f'	UNION SELECT "metric" \n' \
					f'		,CONCAT_WS(\'|\',\'total\',{dimension}) AS "dimension"\n' \
					f'		,SUM("count") AS "value"\n' \
					f'	FROM raw_counts GROUP BY 1,2 \n'

		if f_missing is not None:
			sql +=	f'	UNION SELECT "metric" \n' \
					f'		,CONCAT_WS(\'|\',\'missing\',{dimension}) AS "dimension"\n' \
					f'		,SUM("count") AS "value"\n' \
					f'	FROM raw_counts WHERE "{f_missing}"=1 GROUP BY 1,2 \n'

			missing_clause = 'AND "{}" IS NULL'.format(f_missing)
		else:
			missing_clause = ''

		if f_deleted is not None:
			sql +=	f'	UNION SELECT "metric" \n' \
					f'		,CONCAT_WS(\'|\',\'deleted\',{dimension}) AS "dimension"\n' \
					f'		,SUM("count") AS "value"\n' \
					f'	FROM raw_counts WHERE "{f_deleted}"=1 GROUP BY 1,2 \n'

			deleted_clause = 'AND "{}" IS NULL '.format(f_deleted)
		else:
			deleted_clause = ''


		if f_inactive is not None:
			sql +=	f'	UNION SELECT "metric" \n' \
					f'		,CONCAT_WS(\'|\',\'inactive\',{dimension}) AS "dimension"\n' \
					f'		,SUM("count") AS "value"\n' \
					f'	FROM raw_counts WHERE "{f_inactive}"=1 {deleted_clause} GROUP BY 1,2 \n'

			sql +=	f'	UNION SELECT "metric" \n' \
					f'		,CONCAT_WS(\'|\',\'active\',{dimension}) AS "dimension"\n' \
					f'		,SUM("count") AS "value"\n' \
					f'	FROM raw_counts WHERE "{f_inactive}" IS NULL {deleted_clause} {missing_clause} GROUP BY 1,2 \n'
		else:
			sql +=	f'	UNION SELECT "metric" \n' \
					f'		,CONCAT_WS(\'|\',\'active\',{dimension}) AS "dimension"\n' \
					f'		,SUM("count") AS "value"\n' \
					f'	FROM raw_counts WHERE 1=1 {deleted_clause} {missing_clause} GROUP BY 1,2 \n'


	else:	# status not requested, pass data as is
		sql +=	f'	SELECT "metric", {dimension} AS "dimension", "count" AS "value" FROM raw_counts\n'

	# SQL to left join all dimension combinations with counts (and populate NULL values with 0)
	if do_pad_with_zeros:
		if metric is None:
			metric = f"(SELECT DISTINCT \"metric\" FROM counts LIMIT 1)"
		else:
			metric = f"'{metric}'"
		sql +=	f'),\n'		# counts AS (...)
		sql +=	f'counts_with_zeros AS (\n' \
				f'	SELECT {metric} AS "metric",\n' \
				f'			adc."dimension",\n' \
				f'			COALESCE(c."value",0) AS "value"\n' \
				f'	FROM all_dimension_combinations adc LEFT JOIN counts c\n' \
				f'		ON adc.dimension=c.dimension\n' \
				f')\n'
	else:
		sql +=	f')\n'		# counts AS (...)

	stats_insert(out_file, comment, sql, 'counts_with_zeros' if do_pad_with_zeros else 'counts')


#######################################################


def main():

	global sql_flavor
	global out_file

	if len(sys.argv) < 4:
		print(f"\nUSAGE: python3 {sys.argv[0]}  ./path/to/fields.tsv  pgsql|mysql|sqlite  ./path/to/output.sql  [sqlite_db_path_stub]\n\n")
		sys.exit()

	fields_tsv 	= sys.argv[1]
	sql_flavor 	= sys.argv[2]
	out_file 	= sys.argv[3]

	if len(sys.argv) == 5:
		sqlite_db_path_stub = sys.argv[4]

	if sql_flavor not in ['sqlite','mysql','pgsql']:
		print(f"\nUSAGE: python3 {sys.argv[0]}  ./path/to/fields.tsv  pgsql|mysql|sqlite  ./path/to/output.sql  [sqlite_db_path_stub]\n\n")
		sys.exit()

	fields = []
	with open(fields_tsv) as tsv:
		for row in csv.reader(tsv, delimiter = "\t"):
			f = {}
			i = 0
			f['entity'] 		= row[i+0]
			f['table'] 			= row[i+1]
			f['ord'] 			= row[i+2]
			f['name'] 			= row[i+3]
			f['type'] 			= row[i+4]
			f['nullable']		= row[i+5]
			f['max_length'] 	= row[i+6]
			f['pdk_function'] 	= row[i+7]
			f['xpath'] 			= row[i+8]
			fields.append(f)

	tables = build_pdk_function_dict(fields)


	# always call the header first
	stats_header(out_file)

	# db entity sizes
	stats_db_size(out_file)

	# ingest stats
	stats_ingests(out_file, "program", "TMSId")
	stats_ingests(out_file, "schedule", "scheduleId")
	stats_ingests(out_file, "source", "prgSvcId")
	stats_ingests(out_file, "celebrity", "personId")
	stats_ingests(out_file, "lineup", "lineupId")
	stats_ingests(out_file, "availability", "availabilityId", dimension="catalogName" )
	stats_ingests(out_file, "mapping", "mappingId", dimension="catalogName" )
	stats_ingests(out_file, "descriptortaxonomy", "taxonomyId")
	stats_ingests(out_file, "descriptor", "descriptorId")
	stats_ingests(out_file, "popularity", "popularityId")

	stats_ingests(out_file, "sportsorg", "organizationId")
#	stats_ingests(out_file, "sportsconf", "conferenceId")
	stats_ingests(out_file, "team", "teamId")
#	stats_ingests(out_file, "teambrand", "teamBrandId")
	stats_ingests(out_file, "venue", "venueId")
#	stats_ingests(out_file, "venuebrand", "venueBrandId")
	stats_ingests(out_file, "university", "universityId")

	# update stats
	stats_updates(out_file, "program", "TMSId")
	stats_updates(out_file, "schedule", "scheduleId")
	stats_updates(out_file, "source", "prgSvcId")
	stats_updates(out_file, "celebrity", "personId")
	stats_updates(out_file, "lineup", "lineupId")
	stats_updates(out_file, "availability", "availabilityId", dimension="catalogName" )
	stats_updates(out_file, "mapping", "mappingId", dimension="catalogName" )
	stats_updates(out_file, "descriptortaxonomy", "taxonomyId")
	stats_updates(out_file, "descriptor", "descriptorId")
	stats_updates(out_file, "popularity", "popularityId")

	stats_updates(out_file, "sportsorg", "organizationId")
#	stats_updates(out_file, "sportsconf", "conferenceId")
	stats_updates(out_file, "team", "teamId")
#	stats_updates(out_file, "teambrand", "teamBrandId")
	stats_updates(out_file, "venue", "venueId")
#	stats_updates(out_file, "venuebrand", "venueBrandId")
	stats_updates(out_file, "university", "universityId")




	# === PROGRAMS ===

	# program counts with status
	stats_counts(out_file, "program counts with status", raw_counts_sql="""
	SELECT 
		p."deleted", p."inactive"
		,'programs{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(p."TMSId") AS "count"
	FROM program p
	GROUP BY 1,2,3,4
	"""
		, status=True, f_deleted='deleted', f_inactive='inactive'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|inactive' }]
		, metric='programs{STATUS}'
		)

	# program counts with status by desc language
	stats_counts(out_file, "program counts with status by desc language", raw_counts_sql="""
	SELECT 
		p."deleted", p."inactive", p."desc_lang"
		,'programs{STATUS|desc_lang}' AS "metric"
		,p."desc_lang" AS "dimension"
		,COUNT(p."TMSId") AS "count"
	FROM program p
	GROUP BY 1,2,3,4,5
	"""
		, status=True, f_deleted='deleted', f_inactive='inactive'
		, dims=[
			  { 'dim':'STATUS', 'values':'total|active|deleted|inactive' }
			 ,{ 'dim':'desc_lang', 'field':'desc_lang' }]
		, metric='programs{STATUS|desc_lang}'
		)

	# active program counts by progType
	stats_counts(out_file, "active program counts by progType", raw_counts_sql="""
	SELECT 
		'programs[active]{progType}' AS "metric"
		,p."progType" AS "dimension"
		,COUNT(p."TMSId") AS "count"
	FROM program p
	WHERE p."deleted" IS NULL AND p."inactive" IS NULL
	GROUP BY 1,2
	"""
		)

	# active program counts by progType/subType
	stats_counts(out_file, "active program counts by progType/subType", raw_counts_sql="""
	SELECT 
		'programs[active]{progType|subType}' AS "metric"
		,CONCAT_WS('|',p."progType",p."subType") AS "dimension"
		,COUNT(p."TMSId") AS "count"
	FROM program p
	WHERE p."deleted" IS NULL AND p."inactive" IS NULL
	GROUP BY 1,2
	"""
		)

	# episode series with status
	stats_counts(out_file, "episode series with status", raw_counts_sql="""
	SELECT 
		CASE WHEN p."TMSId" IS NULL THEN 1 ELSE NULL END AS "is_null", p."deleted", p."inactive"
		,'episode[active]_parentseries{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT p."TMSId") AS "count"
	FROM program ep
		LEFT JOIN program p ON ep."connectorId"=p."TMSId"
	WHERE ep."deleted" IS NULL AND ep."inactive" IS NULL
	GROUP BY 1,2,3,4,5
	"""
		, status=True, f_deleted='deleted', f_inactive='inactive', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|inactive|missing' }]
		, metric='episode[active]_parentseries{STATUS}'
		)

	# episode seasons with status
	stats_counts(out_file, "episode seasons with status", raw_counts_sql="""
	SELECT 
		CASE WHEN s."seasonTMSId" IS NULL THEN 1 ELSE NULL END AS "is_null", s."deleted", s."inactive"
		,'episode[active]_parentseasons{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT s."seasonTMSId") AS "count"
	FROM program ep
		LEFT JOIN season s ON s."seasonTMSId"=ep."seasonId"||'-'||ep."connectorId"
	WHERE ep."deleted" IS NULL AND ep."inactive" IS NULL 
		AND ep."seasonId" IS NOT NULL
	GROUP BY 1,2,3,4,5
	"""
		, status=True, f_deleted='deleted', f_inactive='inactive', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|inactive|missing' }]
		, metric='episode[active]_parentseasons{STATUS}'
		)


	# === AVAILABILITIES ===

	stats_counts(out_file, "availabiities by catalog", raw_counts_sql="""
	SELECT 
		'availabilities{catalogName}' AS "metric"
		,a."catalogName" AS "dimension"
		,COUNT(a."availabilityId") AS "count"
	FROM availability a
	GROUP BY 1,2
	"""
		)

	stats_counts(out_file, "availabiities with status by catalog", raw_counts_sql="""
	SELECT 
		a."deleted", a."catalogName"
		,'availabilities{STATUS|catalogName}' AS "metric"
		,a."catalogName" AS "dimension"
		,COUNT(a."availabilityId") AS "count"
	FROM availability a
	GROUP BY 1,2,3
	"""
		, status=True, f_deleted='deleted'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted' }
			,{ 'dim':'catalogName', 'field':'catalogName' }]
		, metric='availabilities{STATUS|catalogName}'
		)

	# availabiities with status by catalog/license
	stats_counts(out_file, "active availabiities with status by catalog/license", raw_counts_sql="""
	SELECT 
		'availabilities[active]{catalogName|license}' AS "metric"
		,CONCAT_WS('|',a."catalogName",av."license") AS "dimension"
		,COUNT(a."availabilityId") AS "count"
	FROM availability a
		JOIN availability_viewingoption av ON a."availabilityId"=av."availabilityId"
	WHERE a."deleted" IS NULL
	GROUP BY 1,2
	"""
		)

	# availabiities with status by catalog/urltype
	stats_counts(out_file, "active availabiities with status by catalog/urltype", raw_counts_sql="""
	SELECT 
		'availabilities[active]{catalogName|urltype}' AS "metric"
		,CONCAT_WS('|',a."catalogName",au."type") AS "dimension"
		,COUNT(a."availabilityId") AS "count"
	FROM availability a
		JOIN availability_url au ON a."availabilityId"=au."availabilityId"
	WHERE a."deleted" IS NULL
	GROUP BY 1,2
	"""
		)

	# availabiities with status by catalog/providerdata
	stats_counts(out_file, "active availabiities with status by catalog/providerdata", raw_counts_sql="""
	SELECT 
		'availabilities[active]{catalogName|providerdata}' AS "metric"
		,CONCAT_WS('|',a."catalogName",ap."key") AS "dimension"
		,COUNT(a."availabilityId") AS "count"
	FROM availability a
		JOIN availability_providerdata ap ON a."availabilityId"=ap."availabilityId"
	WHERE a."deleted" IS NULL
	GROUP BY 1,2
	"""
		)

	# active availabilities by catalog/window (in window, out - expired/future, no window - undef)
	stats_counts(out_file, f"active availabilities by catalog/window", raw_counts_sql="""
	SELECT
		CASE
			WHEN (a."availableFrom" IS NULL OR a."expiresAt" IS NULL) THEN 'undef'
			WHEN ({0} BETWEEN a."availableFrom" AND a."expiresAt") THEN 'in'
			WHEN ({0} > a."expiresAt") THEN 'expired'
			WHEN ({0} < a."availableFrom") THEN 'future'		
		END AS "window"
		,a."catalogName"
		,'availabilities[active]{{catalogName|window}}' AS "metric"
		,COUNT(a."availabilityId") AS "count"
	FROM availability a
	WHERE a."deleted" IS NULL
	GROUP BY 1,2,3
	""".format(today())
			, merge_dims=True
			, dims=[
				 { 'dim':'catalogName', 'metric':'availabilities{catalogName}' }
				,{ 'dim':'window', 'values':'undef|in|expired|future' }]
			, metric='availabilities[active]{catalogName|window}'
			)


	# SQL - active availabilities by catalog with window
	def sql_availabilities_by_catalog_with_windowsize(subdim):
		if (subdim == '1wk'):
			where = 'AND {0} <= ({1})'.format('a."expiresAt"'
				,date_plus_interval('a."availableFrom"', 7, "day"))

		elif (subdim == '1mo'):
			where = 'AND {0} > ({1}) AND {0} <= ({2})'.format('a."expiresAt"'
				,date_plus_interval('a."availableFrom"', 7, "day")
				,date_plus_interval('a."availableFrom"', 1, "month"))

		elif (subdim == '1yr'):
			where = 'AND {0} > ({1}) AND {0} <= ({2})'.format('a."expiresAt"'
				,date_plus_interval('a."availableFrom"', 1, "month")
				,date_plus_interval('a."availableFrom"', 1, "year"))

		elif (subdim == 'longterm'):
			where = 'AND {0} > ({1})'.format('a."expiresAt"'
				,date_plus_interval('a."availableFrom"', 1, "year"))
		else:
			return None

		sql = """
	SELECT 
		'availabilities[active]{{catalogName|windowSize}}' AS "metric"
		,CONCAT_WS('|',a."catalogName",'{0}') AS "dimension"
		,COUNT(a."availabilityId") AS "count"
	FROM availability a
	WHERE a."deleted" IS NULL
		AND a."expiresAt" IS NOT NULL
		AND a."availableFrom" IS NOT NULL
		{1}
	GROUP BY 1,2
		""".format(subdim,where)
		return sql


	# active availabilities by catalog with windowsizein (1wk,1mo,1yr,longterm)
	for subdim in ["1wk","1mo","1yr","longterm"]:
		stats_counts(out_file, f"active availabilities by catalog - windowsize:{subdim}"
				, sql_availabilities_by_catalog_with_windowsize(subdim)
				, dims=[
					 { 'dim':'catalogName', 'metric':'availabilities{catalogName}' }
					,{ 'dim':'windowSize', 'values':f'{subdim}' }]
				, metric='availabilities[active]{catalogName|windowSize}'
				)


	# availability programs with status
	stats_counts(out_file, "availabiity programs with status", raw_counts_sql="""
	SELECT 
		CASE WHEN p."TMSId" IS NULL THEN 1 ELSE NULL END AS "is_null", p."deleted", p."inactive"
		,'availability[active]_programs{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT ai."id") AS "count"
	FROM availability a
		JOIN availability_id ai ON a."availabilityId"=ai."availabilityId"
		LEFT JOIN program p ON ai."id"=p."TMSId"
	WHERE a."deleted" IS NULL 
		AND ai."type"='tmsId'
	GROUP BY 1,2,3,4,5
	"""
		, status=True, f_deleted='deleted', f_inactive='inactive', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|inactive|missing' }]
		, metric='availability[active]_programs{STATUS}'
		)

	# availability sources with status
	stats_counts(out_file, "availability sources with status", raw_counts_sql="""
	SELECT 
		CASE WHEN s."prgSvcId" IS NULL THEN 1 ELSE NULL END AS "is_null", s."deleted", s."inactive"
		,'availability[active]_sources{{STATUS}}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT ai."id") AS "count"
	FROM availability a
		JOIN availability_id ai ON a."availabilityId"=ai."availabilityId"
		LEFT JOIN source s ON {0}=s."prgSvcId"
	WHERE a."deleted" IS NULL 
		AND ai."type"='prgSvcId'
	GROUP BY 1,2,3,4,5
	""".format(cast_as('ai."id"','int'))

		, status=True, f_deleted='deleted', f_inactive='inactive', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|inactive|missing' }]
		, metric='availability[active]_sources{STATUS}'			 
		)	


	# active availabiity programs by catalog
	stats_counts(out_file, "active availabiity programs by catalog", raw_counts_sql="""
	SELECT 
		'availability[active]_programs{catalogName}' AS "metric"
		,a."catalogName" AS "dimension"
		,COUNT(DISTINCT ai.id) AS "count"
	FROM availability a
		JOIN availability_id ai ON a."availabilityId"=ai."availabilityId"
	WHERE a."deleted" IS NULL
		AND ai."type"='tmsId'
	GROUP BY 1,2
	"""
		)



	# === MAPPINGS ===

	# mappings by catalog
	stats_counts(out_file, "mappings by catalog", raw_counts_sql="""
	SELECT 
		'mappings{catalogName}' AS "metric"
		,m."catalogName" AS "dimension"
		,COUNT(m."mappingId") AS "count"
	FROM mapping m
	GROUP BY 1,2
	"""
		)

	# mappings with entity status by catalog
	stats_counts(out_file, "mappings with status by catalog", raw_counts_sql="""
	SELECT 
		m."deleted", m."catalogName"
		,'mappings{STATUS|catalogName}' AS "metric"
		,m."catalogName" AS "dimension"
		,COUNT(m."mappingId") AS "count"
	FROM mapping m
	GROUP BY 1,2,3
	"""
		, status=True, f_deleted='deleted'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted' }
			,{ 'dim':'catalogName', 'field':'catalogName' }]
		, metric='mappings{STATUS|catalogName}'
		)

	# active mappings by catalog/mapping status
	stats_counts(out_file, "active mappings by catalog/mapping status", raw_counts_sql="""
	SELECT 
		 m."catalogName", m."status"
		,'mappings[active]{catalogName|status}' AS "metric"
		,CONCAT_WS('|',m."catalogName",m."status") AS "dimension"
		,COUNT(m."mappingId") AS "count"
	FROM mapping m
	WHERE m."deleted" IS NULL
	GROUP BY 1,2
	"""
		, dims=[
			 { 'dim':'catalogName', 'field':'catalogName' }
			,{ 'dim':'status', 'field':'status' }]
		, metric='mappings[active]{catalogName|status}'
		)

	# active/Mapped mappings by catalog/window (in window, out - expired/future, no window - undef)
	stats_counts(out_file, f"active/Mapped mappings by catalog/window", raw_counts_sql="""
	SELECT
		CASE
			WHEN (m."avail_start" IS NULL OR m."avail_end" IS NULL) THEN 'undef'
			WHEN ({0} BETWEEN m."avail_start" AND m."avail_end") THEN 'in'
			WHEN ({0} > m."avail_end") THEN 'expired'
			WHEN ({0} < m."avail_start") THEN 'future'		
		END AS "window"
		,m."catalogName"
		,'mappings[active,Mapped]{{catalogName|window}}' AS "metric"
		,COUNT(m."mappingId") AS "count"
	FROM mapping m
	WHERE m."deleted" IS NULL 
		AND m."status"='Mapped'
	GROUP BY 1,2,3
	""".format(today())
			, merge_dims=True
			, dims=[
				 { 'dim':'catalogName', 'metric':'mappings{catalogName}' }
				,{ 'dim':'window', 'values':'undef|in|expired|future' }]
			, metric='mappings[active,Mapped]{catalogName|window}'
			)

	# active/ToBeMapped mappings by catalog/window (in window, out - expired/future, no window - undef)
	stats_counts(out_file, f"active/ToBeMapped mappings by catalog/window", raw_counts_sql="""
	SELECT
		CASE
			WHEN (m."avail_start" IS NULL OR m."avail_end" IS NULL) THEN 'undef'
			WHEN ({0} BETWEEN m."avail_start" AND m."avail_end") THEN 'in'
			WHEN ({0} > m."avail_end") THEN 'expired'
			WHEN ({0} < m."avail_start") THEN 'future'		
		END AS "window"
		,m."catalogName"
		,'mappings[active,ToBeMapped]{{catalogName|window}}' AS "metric"
		,COUNT(m."mappingId") AS "count"
	FROM mapping m
	WHERE m."deleted" IS NULL 
		AND m."status"='ToBeMapped'
	GROUP BY 1,2,3
	""".format(today())
			, merge_dims=True
			, dims=[
				 { 'dim':'catalogName', 'metric':'mappings{catalogName}' }
				,{ 'dim':'window', 'values':'undef|in|expired|future' }]
			, metric='mappings[active,ToBeMapped]{catalogName|window}'
			)

	# active/ToBeMapped mappings by catalog/reason
	stats_counts(out_file, f"active/ToBeMapped mappings by catalog/reason", raw_counts_sql="""
	SELECT
		CONCAT_WS('|',m."catalogName",COALESCE(m."message_reason",' NULL')) AS "dimension"
		,'mappings[active,ToBeMapped]{catalogName|reason}' AS "metric"
		,COUNT(m."mappingId") AS "count"
	FROM mapping m
	WHERE m."deleted" IS NULL 
		AND m."status"='ToBeMapped'
	GROUP BY 1,2
	"""
			)

	# active/Unmappable mappings by catalog/reason
	stats_counts(out_file, f"active/Unmappable mappings by catalog/reason", raw_counts_sql="""
	SELECT
		CONCAT_WS('|',m."catalogName",COALESCE(m."message_reason",' NULL')) AS "dimension"
		,'mappings[active,Unmappable]{catalogName|reason}' AS "metric"
		,COUNT(m."mappingId") AS "count"
	FROM mapping m
	WHERE m."deleted" IS NULL 
		AND m."status"='Unmappable'
	GROUP BY 1,2
	"""
			)	

	# mapping programs with status
	stats_counts(out_file, "mapping programs with status", raw_counts_sql="""
	SELECT 
		CASE WHEN p."TMSId" IS NULL THEN 1 ELSE NULL END AS is_null, p."deleted", p."inactive"
		,'mapping[active]_programs{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT mi."id") AS "count"
	FROM mapping m
		JOIN mapping_id mi ON m."mappingId"=mi."mappingId"
		LEFT JOIN program p ON mi."id"=p."TMSId"
	WHERE m."deleted" IS NULL
		AND mi."type"='TMSId'
	GROUP BY 1,2,3,4,5
	"""
		, status=True, f_deleted='deleted', f_inactive='inactive', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|inactive|missing' }]
		, metric='mapping[active]_programs{STATUS}'
		)

	# active mapping programs by catalog
	stats_counts(out_file, "active mapping programs by catalog", raw_counts_sql="""
	SELECT 
		'mapping[active]_programs{catalogName}' AS "metric"
		,m."catalogName" AS "dimension"
		,COUNT(DISTINCT mi."id") AS "count"
	FROM mapping m
		JOIN mapping_id mi ON m."mappingId"=mi."mappingId"
	WHERE m."deleted" IS NULL
		AND mi."type"='TMSId'
	GROUP BY 1,2
	"""
		)



	# === SCHEDULES ===

	# schedule sources
	stats_counts(out_file, "schedule sources", raw_counts_sql="""
	SELECT 
		'schedule_sources' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT h."prgSvcId") AS "count"
	FROM schedule h
	GROUP BY 1,2
	"""
		)

	# schedule dates (not scheduleDays)
	stats_counts(out_file, "schedule dates", raw_counts_sql="""
	SELECT 
		CASE 
			WHEN h."date" >= {0} THEN 'forward'
			WHEN h."date" < {0} THEN 'backward'
		END AS "dimension"
		,'schedule_dates{{horizon}}' AS "metric"
		,COUNT(DISTINCT h."date") AS "count"
	FROM schedule h
	GROUP BY 1,2
	""".format(today())
		)

	if False:
		# schedule(day)s by callsign
		stats_counts(out_file, "schedule(day)s by callsign", raw_counts_sql="""
		SELECT 
			s."callSign" AS "dimension"
			,'schedules{callSign}' AS "metric"
			,COUNT(h."scheduleId") AS "count"
		FROM schedule h
			JOIN source s ON h."prgSvcId"=s."prgSvcId"
		WHERE s."deleted" IS NULL
		GROUP BY 1,2
		"""
			)

	# schedule events
	stats_counts(out_file, "schedule events", raw_counts_sql="""
	SELECT 
		CASE 
			WHEN h."date" >= {0} THEN 'forward'
			WHEN h."date" < {0} THEN 'backward'
		END AS "dimension"
		,'schedule_events{{horizon}}' AS "metric"
		,COUNT(he."eventId") AS "count"
	FROM schedule h
		JOIN schedule_event he ON h."scheduleId"=he."scheduleId"
	GROUP BY 1,2
	""".format(today())
		)


	# SQL - schedule event/program types
	def sql_schedule_events_by_ptype(subdim):
		if (subdim == 'forward'):
			where = 'h."date" >= {0}'.format(today())
		elif (subdim == 'next7d'):
			where = 'h."date" BETWEEN {0} AND ({1})'.format(today(), date_plus_interval(today(),+7,'day'))
		elif (subdim == 'last7d'):
			where = 'h."date" BETWEEN ({1}) AND {0}'.format(today(), date_plus_interval(today(),-7,'day'))
		elif (subdim == 'today'):
			where = 'h."date" = {0}'.format(today())
		else:
			return None

		sql = """
	SELECT 
		CASE 
			WHEN (p."progType"='TBA') THEN 'tba'
			WHEN (p."progType"='Off Air') THEN 'offair'
			ELSE 'generic'
		END AS "dimension"
		,'schedule[{0}]_events{{pType}}' AS "metric"
		,COUNT(he."TMSId") AS "count"
	FROM schedule h
		JOIN schedule_event he ON h."scheduleId"=he."scheduleId"
		JOIN program p ON he."TMSId"=p."TMSId"
	WHERE {1}
	GROUP BY 1,2
		""".format(subdim,where)
		return sql


	# schedule event/program types
	for horizon in ['today','next7d','last7d','forward']:
		stats_counts(out_file, f"schedule events by pType - {horizon}",
			sql_schedule_events_by_ptype(horizon) )


	# schedule programs with status - 1d
	stats_counts(out_file, "schedule programs with status - 1d", raw_counts_sql="""
	SELECT 
		CASE WHEN p."TMSId" IS NULL THEN 1 ELSE NULL END AS is_null, p."deleted", p."inactive"
		,'schedule[now-1d]_programs{{STATUS}}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT he."TMSId") AS "count"
	FROM schedule h
		JOIN schedule_event he ON h."scheduleId"=he."scheduleId"
		LEFT JOIN program p ON he."TMSId"=p."TMSId"
	WHERE h."date" BETWEEN {0} AND ({1})
	GROUP BY 1,2,3,4,5
	""".format(today(),date_plus_interval(today(),1,'day'))

		, status=True, f_deleted='deleted', f_inactive='inactive', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|inactive|missing' }]
		, metric='schedule[now-1d]_programs{STATUS}'
		)

	# schedule programs with status - forward
	stats_counts(out_file, "schedule programs with status - forward", raw_counts_sql="""
	SELECT 
		CASE WHEN p."TMSId" IS NULL THEN 1 ELSE NULL END AS is_null, p."deleted", p."inactive"
		,'schedule[forward]_programs{{STATUS}}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT he."TMSId") AS "count"
	FROM schedule h
		JOIN schedule_event he ON h."scheduleId"=he."scheduleId"
		LEFT JOIN program p ON he."TMSId"=p."TMSId"
	WHERE h."date" >= {0}
	GROUP BY 1,2,3,4,5
	""".format(today())

		, status=True, f_deleted='deleted', f_inactive='inactive', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|inactive|missing' }]
		, metric='schedule[forward]_programs{STATUS}'
		)

	# schedule sources with status - forward
	stats_counts(out_file, "schedule sources with status - forward", raw_counts_sql="""
	SELECT 
		CASE WHEN s."prgSvcId" IS NULL THEN 1 ELSE NULL END AS is_null, s."deleted", s."inactive"
		,'schedule[forward]_sources{{STATUS}}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT h."prgSvcId") AS "count"
	FROM schedule h
		LEFT JOIN source s ON h."prgSvcId"=s."prgSvcId"
	WHERE h."date" >= {0}
	GROUP BY 1,2,3,4,5
	""".format(today())

		, status=True, f_deleted='deleted', f_inactive='inactive', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|inactive|missing' }]
		, metric='schedule[forward]_sources{STATUS}'
		)


	def sql_schedule_discontinuities_by_source_and_midnight(subdim):
		if (subdim == 'forward'):
			where = 'h."date" >= {0}'.format(today())
		elif (subdim == 'today'):
			where = 'h."date" BETWEEN {0} AND {1}'.format(
				 date_plus_interval(today(),-1,'day')
				,date_plus_interval(today(),+1,'day'))
		else:
			return None

		if sql_flavor=='pgsql':
			ev_start = '(SUBSTRING(he."scheduleId", POSITION(\'-\' IN he."scheduleId")+1)||\' \'||he.time)::TIMESTAMP'
			ev_end	 = '(SUBSTRING(he."scheduleId", POSITION(\'-\' IN he."scheduleId")+1)||\' \'||he.time)::TIMESTAMP + SUBSTRING(he.dur FROM 3 FOR 6)::INTERVAL'
		elif sql_flavor=='mysql':
			ev_start = 'CAST(SUBSTRING(he."scheduleId", POSITION(\'-\' IN he."scheduleId")+1) AS DATETIME)'
			ev_end 	 = 'ADDTIME(CAST(SUBSTRING(he."scheduleId", POSITION(\'-\' IN he."scheduleId")+1) AS DATETIME), CAST(CONCAT_WS(\':\',SUBSTRING(he."dur" FROM 3 FOR 2),SUBSTRING(he."dur" FROM 6 FOR 2)) AS TIME))'
		elif sql_flavor=='sqlite':
			ev_start = 'DATETIME(SUBSTR(he."scheduleId", INSTR(he."scheduleId",\'-\')+1)||\' \'||he.time)'
			ev_end 	 = 'DATETIME(SUBSTR(he."scheduleId", INSTR(he."scheduleId",\'-\')+1)||\' \'||he.time, SUBSTR(he."dur",3,2)||\':\'||SUBSTR(he."dur",6,2))'


		sql = """
WITH events AS (
	SELECT he."eventId"
		,h."prgSvcId"
		,{2} AS "start"
		,{3} AS "end"
	FROM schedule_event he
		JOIN "schedule" h ON he."scheduleId"=h."scheduleId"
	WHERE {1}
	ORDER BY 2,3
), events_with_next AS (
	SELECT "eventId", "prgSvcId", "start", "end"
		,LEAD("start") OVER (PARTITION BY "prgSvcId") AS "next_start"
	FROM events
), events_with_gap AS (
	SELECT "eventId", "prgSvcId"
		,"start", "end", "next_start"
		,{6}-{7} AS "gap"
		,CASE WHEN {4}={5} THEN 0 ELSE 1 END AS "midnight"
	FROM events_with_next
), counts AS (
	SELECT
		'schedule[{0}]_discontinuities{{prgSvcId|midnight}}' AS "metric"
		,CONCAT_WS('|',"prgSvcId","midnight") AS "dimension"
		,COUNT("eventId") AS "value"
	FROM events_with_gap 
	WHERE "gap" != 0 
	GROUP BY 1,2
	UNION SELECT 
		'schedule[{0}]_discontinuities{{prgSvcId|midnight}}' AS "metric",'|0' AS "dimension", 0 AS "value"
	UNION SELECT 
		'schedule[{0}]_discontinuities{{prgSvcId|midnight}}' AS "metric",'|1' AS "dimension", 0 AS "value"
)
""".format(subdim, where		# 0,1
		,ev_start, ev_end		# 2,3
		,cast_as('"start"','date'), cast_as('"end"','date')			# 4,5
		,epoch('COALESCE("next_start","end")'), epoch('"end"'))		# 6,7

		return sql


	# schedule discontinuities by source/midnight
	for horizon in ['today','forward']:
		stats_insert(out_file, f"schedule discontinuities by source/midnight - {horizon}",
			sql_schedule_discontinuities_by_source_and_midnight(horizon), 'counts' )


	# === SOURCES ===

	# source counts with entity status
	stats_counts(out_file, "source counts with entity status", raw_counts_sql="""
	SELECT 
		s."deleted", s."inactive"
		,'sources{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(s."prgSvcId") AS "count"
	FROM source s
	GROUP BY 1,2,3,4
	"""
		, status=True, f_deleted='deleted', f_inactive='inactive'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|inactive' }]
		, metric='sources{STATUS}'
		)

	# active source counts by editing status/type
	stats_counts(out_file, "active source counts by editing status/type", raw_counts_sql="""
	SELECT 
		'sources[active]{status|type}' AS "metric"
		,CONCAT_WS('|',s."status",s."type") AS "dimension"
		,COUNT(s."prgSvcId") AS "count"
	FROM source s
	WHERE s."deleted" IS NULL AND s.inactive IS NULL
	GROUP BY 1,2
	"""
		)	

	# active source counts by source type
	stats_counts(out_file, "active source counts by source type", raw_counts_sql="""
	SELECT 
		'sources[active]{type}' AS "metric"
		,s."type" AS "dimension"
		,COUNT(s."prgSvcId") AS "count"
	FROM source s
	WHERE s."deleted" IS NULL AND s.inactive IS NULL
	GROUP BY 1,2
	"""
		)

	# active source counts by ed language
	stats_counts(out_file, "active source counts by ed language", raw_counts_sql="""
	SELECT 
		'sources[active]{edLang}' AS "metric"
		,sel."edLang" AS "dimension"
		,COUNT(DISTINCT s."prgSvcId") AS "count"
	FROM source s
		JOIN source_edlang sel on s."prgSvcId"=sel."prgSvcId"
	WHERE s."deleted" IS NULL AND s."inactive" IS NULL
	GROUP BY 1,2
	"""
		)		

	# active source counts by country
	stats_counts(out_file, "active source counts by country", raw_counts_sql="""
	SELECT 
		'sources[active]{country}' AS "metric"
		,sc."country" AS "dimension"
		,COUNT(DISTINCT s."prgSvcId") AS "count"
	FROM source s
		JOIN source_country sc on s."prgSvcId"=sc."prgSvcId"
	WHERE s."deleted" IS NULL AND s."inactive" IS NULL
	GROUP BY 1,2
	"""
		)	

	# === LINEUPS ===

	# === CELEBRITIES ===

	# celebrity counts with entity status
	stats_counts(out_file, "celebrity counts with entity status", raw_counts_sql="""
	SELECT 
		c."deleted"
		,'celebrities{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(c."personId") AS "count"
	FROM celebrity c
	GROUP BY 1,2,3
	"""
		, status=True, f_deleted='deleted'
		)

	# active celebrity name counts
	stats_counts(out_file, "active celebrity names", raw_counts_sql="""
	SELECT 
		'celebrity[active]_names' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT cn."nameId") AS "count"
	FROM celebrity c
		JOIN celebrity_name cn ON c."personId"=cn."personId"
	WHERE c."deleted" IS NULL
	GROUP BY 1,2
	"""
		)

	# active celebrity names by language
	stats_counts(out_file, "active celebrity names by language", raw_counts_sql="""
	SELECT 
		'celebrity[active]_names{language}' AS "metric"
		,cn."lang" AS "dimension"
		,COUNT(DISTINCT cn."nameId") AS "count"
	FROM celebrity c
		JOIN celebrity_name cn ON c."personId"=cn."personId"
	WHERE c."deleted" IS NULL
	GROUP BY 1,2
	"""
		)	

	# active celebrities by type
	stats_counts(out_file, "active celebrities by type", raw_counts_sql="""
	SELECT 
		'celebrities[active]{type}' AS "metric"
		,c."type" AS "dimension"
		,COUNT(c."personId") AS "count"
	FROM celebrity c
	WHERE c."deleted" IS NULL
	GROUP BY 1,2
	"""
		)

	# active celebrities by image category
	stats_counts(out_file, "active celebrities by image category", raw_counts_sql="""
	SELECT 
		'celebrities[active]{imageCategory}' AS "metric"
		,ca."category" AS "dimension"
		,COUNT(DISTINCT c."personId") AS "count"
	FROM celebrity c
		JOIN celebrity_asset ca ON c."personId"=ca."personId"
	WHERE c."deleted" IS NULL
	GROUP BY 1,2
	"""
		)

	# program cast celebrities with status
	stats_counts(out_file, "program cast celebrities with status", raw_counts_sql="""
	SELECT 
		CASE WHEN c."personId" IS NULL THEN 1 ELSE NULL END AS "is_null", c."deleted"
		,'program[active,cast]_celebrities{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT pct."personId") AS "count"
	FROM program p
		JOIN program_cast pct ON p."TMSId"=pct."TMSId"
		LEFT JOIN celebrity c ON pct."personId"=c."personId"
	WHERE p."deleted" IS NULL
	GROUP BY 1,2,3,4
	"""
		, status=True, f_deleted='deleted', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|missing' }]
		, metric='program[active,cast]_celebrities{STATUS}'
		)

	# program crew celebrities with status
	stats_counts(out_file, "program crew celebrities with status", raw_counts_sql="""
	SELECT 
		CASE WHEN c."personId" IS NULL THEN 1 ELSE NULL END AS "is_null", c."deleted"
		,'program[active,crew]_celebrities{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT pcw."personId") AS "count"
	FROM program p
		JOIN program_crew pcw ON p."TMSId"=pcw."TMSId"
		LEFT JOIN celebrity c ON pcw."personId"=c."personId"
	WHERE p."deleted" IS NULL
	GROUP BY 1,2,3,4
	"""
		, status=True, f_deleted='deleted', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|missing' }]
		, metric='program[active,crew]_celebrities{STATUS}'
		)


	# === DESCRIPTORS ===

	# descriptor taxonomies with entity status
	stats_counts(out_file, "descriptor taxonomy counts with entity status", raw_counts_sql="""
	SELECT 
		d."deleted"
		,'descriptortaxonomies{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(d."taxonomyId") AS "count"
	FROM descriptortaxonomy d
	GROUP BY 1,2,3
	"""
		, status=True, f_deleted='deleted'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted' }]
		, metric='descriptortaxonomies{STATUS}'
		)

	# descriptor counts with entity status
	stats_counts(out_file, "descriptor counts with entity status", raw_counts_sql="""
	SELECT 
		d."deleted"
		,'descriptors{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(d."descriptorId") AS "count"
	FROM descriptor d
	GROUP BY 1,2,3
	"""
		, status=True, f_deleted='deleted'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted' }]
		, metric='descriptors{STATUS}'
		)

	# active descriptors by type
	stats_counts(out_file, "active descriptors by type", raw_counts_sql="""
	SELECT 
		'descriptors[active]{typeName}' AS "metric"
		,d."typeName" AS "dimension"
		,COUNT(d."descriptorId") AS "count"
	FROM descriptor d
	WHERE d."deleted" IS NULL
	GROUP BY 1,2
	"""
		)

	# descriptor parents with status
	stats_counts(out_file, "descriptor parents with status", raw_counts_sql="""
	SELECT 
		CASE WHEN dp."descriptorId" IS NULL THEN 1 ELSE NULL END AS "is_null", dp."deleted"
		,'descriptor[active]_parents{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT d."parent_descriptorId") AS "count"
	FROM descriptor d
		LEFT JOIN descriptor dp ON dp."descriptorId"=d."parent_descriptorId"
	WHERE d."deleted" IS NULL
	GROUP BY 1,2,3,4
	"""
		, status=True, f_deleted='deleted', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|missing' }]
		, metric='descriptor[active]_parents{STATUS}'
		)

	# program descriptors with status
	stats_counts(out_file, "program descriptors with status", raw_counts_sql="""
	SELECT 
		CASE WHEN d."descriptorId" IS NULL THEN 1 ELSE NULL END AS "is_null", d."deleted"
		,'program[active]_descriptors{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT pd."descriptorId") AS "count"
	FROM program p
		JOIN program_descriptor pd ON p."TMSId"=pd."TMSId"
		LEFT JOIN descriptor d ON pd."descriptorId"=d."descriptorId"
	WHERE p."deleted" IS NULL AND p."inactive" IS NULL
	GROUP BY 1,2,3,4
	"""
		, status=True, f_deleted='deleted', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|missing' }]
		, metric='program[active]_descriptors{STATUS}'
		)


	# === POPULARITY ===

	# popularity counts with entity status
	stats_counts(out_file, "popularity counts with entity status", raw_counts_sql="""
	SELECT 
		p."deleted"
		,'popularity{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(p."popularityId") AS "count"
	FROM popularity p
	GROUP BY 1,2,3
	"""
		, status=True, f_deleted='deleted'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted' }]
		, metric='popularity{STATUS}'
		)

	# === ORGANIZATIONS ===

	# organization counts with entity status
	stats_counts(out_file, "organization counts with entity status", raw_counts_sql="""
	SELECT 
		o."deleted"
		,'organizations{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(o."organizationId") AS "count"
	FROM sportsorg o
	GROUP BY 1,2,3
	"""
		, status=True, f_deleted='deleted'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted' }]
		, metric='organizations{STATUS}'
		)

	# conference counts
	stats_counts(out_file, "conference counts", raw_counts_sql="""
	SELECT 
		'conferences' AS "metric"
		,NULL AS "dimension"
		,COUNT(c."conferenceId") AS "count"
	FROM sportsconf c
	GROUP BY 1,2
	"""
		)

	# active official organizations by sport/genre
	stats_counts(out_file, "active official organization counts by sport/genre", raw_counts_sql="""
	SELECT 
		'organizations[active,official]{genre}' AS "metric"
		,o."genre" AS "dimension"
		,COUNT(o."organizationId") AS "count"
	FROM sportsorg o
		WHERE o."deleted" IS NULL
			AND o."official"=1
	GROUP BY 1,2
	"""
		)

	# === TEAMS ===

	# team counts with entity status
	stats_counts(out_file, "team counts with entity status", raw_counts_sql="""
	SELECT 
		t."deleted"
		,'teams{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(t."teamId") AS "count"
	FROM team t
	GROUP BY 1,2,3
	"""
		, status=True, f_deleted='deleted'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted' }]
		, metric='teams{STATUS}'
		)

	# active teambrand teams with status
	stats_counts(out_file, "active teambrand teams with entity status", raw_counts_sql="""
	SELECT 
		CASE WHEN t."teamId" IS NULL THEN 1 ELSE NULL END AS "is_null", t."deleted"
		,'teambrand[active]_teams{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT tb."teamId") AS "count"
	FROM teambrand tb
		LEFT JOIN team t ON tb."teamId"=t."teamId"
	WHERE tb."deleted" IS NULL AND tb."active"=1
	GROUP BY 1,2,3,4
	"""
		, status=True, f_deleted='deleted', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|missing' }]
		, metric='teambrand[active]_teams{STATUS}'
		)	

	# teambrand counts with status by active flag
	stats_counts(out_file, "teambrand counts with status by active flag", raw_counts_sql="""
	SELECT 
		tb."deleted", tb."active" AS "active_brand"
		,'teambrands{STATUS|active_brand}' AS "metric"
		,tb."active" AS "dimension"
		,COUNT(tb."teamBrandId") AS "count"
	FROM teambrand tb
	GROUP BY 1,2,3
	"""
		, status=True, f_deleted='deleted'
		, dims=[
			  { 'dim':'STATUS', 'values':'total|active|deleted' }
			 ,{ 'dim':'active_brand', 'field':'active_brand' }]
		, metric='teambrands{STATUS|active_brand}'
		)

	# program teambrands with status by active flag
	stats_counts(out_file, "program teambrands with status by active flag", raw_counts_sql="""
	SELECT 
		CASE WHEN tb."teamBrandId" IS NULL THEN 1 ELSE NULL END AS "is_null", tb."deleted", tb."active" AS "active_brand"
		,'program[active]_teambrands{STATUS|active_brand}' AS "metric"
		,tb."active" AS "dimension"
		,COUNT(DISTINCT pst."teamBrandId") AS "count"
	FROM program p
		JOIN program_spteam pst ON p."TMSId"=pst."TMSId"
		LEFT JOIN teambrand tb ON pst."teamBrandId"=tb."teamBrandId"
	WHERE p."deleted" IS NULL
	GROUP BY 1,2,3,4,5
	"""
		, status=True, f_deleted='deleted', f_missing='is_null'
		, dims=[
			  { 'dim':'STATUS', 'values':'total|active|deleted|missing' }
			 ,{ 'dim':'active_brand', 'field':'active_brand' }]
		, metric='program[active]_teambrands{STATUS|active_brand}'
		)

	# organization teambrands with status by active flag
	stats_counts(out_file, "organization teambrands with status by active flag", raw_counts_sql="""
	SELECT 
		CASE WHEN tb."teamBrandId" IS NULL THEN 1 ELSE NULL END AS "is_null", tb."deleted", tb."active" AS "active_brand"
		,'organization[active]_teambrands{STATUS|active_brand}' AS "metric"
		,tb."active" AS "dimension"
		,COUNT(DISTINCT sotb."teamBrandId") AS "count"
	FROM sportsorg so
		JOIN sportsorg_teambrand sotb ON so."organizationId"=sotb."organizationId"
		LEFT JOIN teambrand tb ON sotb."teamBrandId"=tb."teamBrandId"
	WHERE so."deleted" IS NULL
	GROUP BY 1,2,3,4,5
	"""
		, status=True, f_deleted='deleted', f_missing='is_null'
		, dims=[
			  { 'dim':'STATUS', 'values':'total|active|deleted|missing' }
			 ,{ 'dim':'active_brand', 'field':'active_brand' }]
		, metric='organization[active]_teambrands{STATUS|active_brand}'
		)


	# === VENUES ===

	# venue counts with entity status
	stats_counts(out_file, "venue counts with entity status", raw_counts_sql="""
	SELECT 
		v."deleted"
		,'venues{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(v."venueId") AS "count"
	FROM venue v
	GROUP BY 1,2,3
	"""
		, status=True, f_deleted='deleted'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted' }]
		, metric='venues{STATUS}'
		)

	# active venuebrand venues with status
	stats_counts(out_file, "active venuebrand venues with entity status", raw_counts_sql="""
	SELECT 
		CASE WHEN v."venueId" IS NULL THEN 1 ELSE NULL END AS "is_null", v."deleted"
		,'venuebrand[active]_venues{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT vb."venueId") AS "count"
	FROM venuebrand vb
		LEFT JOIN venue v ON vb."venueId"=v."venueId"
	WHERE vb."deleted" IS NULL AND vb."active"=1
	GROUP BY 1,2,3,4
	"""
		, status=True, f_deleted='deleted', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|missing' }]
		, metric='venuebrand[active]_venues{STATUS}'
		)	

	# active team venues with status
	stats_counts(out_file, "active team venues with entity status", raw_counts_sql="""
	SELECT 
		CASE WHEN v."venueId" IS NULL THEN 1 ELSE NULL END AS "is_null", v."deleted"
		,'team[active]_venues{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT t."venueId") AS "count"
	FROM team t
		LEFT JOIN venue v ON t."venueId"=v."venueId"
	WHERE t."deleted" IS NULL
	GROUP BY 1,2,3,4
	"""
		, status=True, f_deleted='deleted', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|missing' }]
		, metric='team[active]_venues{STATUS}'
		)		

	# venuebrand counts with status by active flag
	stats_counts(out_file, "venuebrand counts with status by active flag", raw_counts_sql="""
	SELECT 
		vb."deleted", vb."active" AS "active_brand"
		,'venuebrands{STATUS|active_brand}' AS "metric"
		,vb."active" AS "dimension"
		,COUNT(vb."venueBrandId") AS "count"
	FROM venuebrand vb
	GROUP BY 1,2,3
	"""
		, status=True, f_deleted='deleted'
		, dims=[
			  { 'dim':'STATUS', 'values':'total|active|deleted' }
			 ,{ 'dim':'active_brand', 'field':'active_brand' }]
		, metric='venuebrands{STATUS|active_brand}'
		)

	# active program venuebrands with status by active flag
	stats_counts(out_file, "program venuebrands with status by active flag", raw_counts_sql="""
	SELECT 
		CASE WHEN vb."venueBrandId" IS NULL THEN 1 ELSE NULL END AS "is_null", vb."deleted", vb."active" AS "active_brand"
		,'program[active]_venuebrands{STATUS|active_brand}' AS "metric"
		,vb."active" AS "dimension"
		,COUNT(DISTINCT psi."venueBrandId") AS "count"
	FROM program p
		JOIN program_spinfo psi ON p."TMSId"=psi."TMSId"
		LEFT JOIN venuebrand vb ON psi."venueBrandId"=vb."venueBrandId"
	WHERE p."deleted" IS NULL
	GROUP BY 1,2,3,4,5
	"""
		, status=True, f_deleted='deleted', f_missing='is_null'
		, dims=[
			  { 'dim':'STATUS', 'values':'total|active|deleted|missing' }
			 ,{ 'dim':'active_brand', 'field':'active_brand' }]
		, metric='program[active]_venuebrands{STATUS|active_brand}'
		)


	# === UNIVERSITIES ===

	# university counts with entity status
	stats_counts(out_file, "university counts with entity status", raw_counts_sql="""
	SELECT 
		u."deleted"
		,'universities{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(u."universityId") AS "count"
	FROM university u
	GROUP BY 1,2,3
	"""
		, status=True, f_deleted='deleted'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted' }]
		, metric='universities{STATUS}'
		)

	# active team universities with status
	stats_counts(out_file, "active team universities with entity status", raw_counts_sql="""
	SELECT 
		CASE WHEN u."universityId" IS NULL THEN 1 ELSE NULL END AS "is_null", u."deleted"
		,'team[active]_universities{STATUS}' AS "metric"
		,NULL AS "dimension"
		,COUNT(DISTINCT t."universityId") AS "count"
	FROM team t
		LEFT JOIN university u ON t."universityId"=u."universityId"
	WHERE t."deleted" IS NULL
	GROUP BY 1,2,3,4
	"""
		, status=True, f_deleted='deleted', f_missing='is_null'
		, dims=[
			 { 'dim':'STATUS', 'values':'total|active|deleted|missing' }]
		, metric='team[active]_universities{STATUS}'
		)		




if __name__ == "__main__":
    main()

